@@ -14,25 +14,6 @@
 typedef int WOTYPE, WITYPE;
 typedef struct fontst FONT;
 
-/* flags for me_flags
- */
-#define ME_FONT		0x3	/* mask for font */
-#define ME_VER		0x4	/* node is divided vertically */
-#define ME_HOR		0x8	/* node divided horizontally */
-#define ME_DIV		0xc	/* node is divided */
-#define ME_NSEL		0x10	/* This node can't be selected */
-#define ME_POPUP	0x20	/* use in pop-up mode */
-#define ME_FREC		0x40	/* me_cap can be free()'d */
-#define ME_FREN		0x80	/* the node itself can be free()'d */
-#define ME_OPEN		0x100	/* the node is open */
-#define ME_ISUB		0x200	/* has an indivisible submenu */
-#define ME_FREE		0x400	/* free standing node */
-#define ME_BGLINE	0x800	/* draw lines in background colour */
-#define ME_NOSCALE	0x1000	/* don't scale submenu */
-#define ME_CREL		0x2000	/* cursor relative submenu */
-#define ME_REDRAW	0x4000  /* flag to force menu redrawing */
-#define ME_LEFT		0x8000	/* left justify caption rather than centre it */
-
 #define ISCLEAR(m)	(((m)->me_cap == NULL) || \
 			(((m)->me_flags & (ME_OPEN|ME_ISUB)) == ME_OPEN))
 
