@@ -327,11 +327,13 @@ char *mname;
 	    getnum(fp, &mnode->me_colour,',')		||
 	    getlongnum(fp, &mnode->me_fg_colour,',')	||
 	    getlongnum(fp, &mnode->me_bg_colour,',')	||
-	    skipnull(fp,',')				||
+	    skipnull(fp,',') /* me_save */		||
 	    getnodenum(fp, &mnode->me_topleft,',')	||
 	    getnodenum(fp, &mnode->me_botrite,',')	||
-	    skipnull(fp,',')				||
-	    skipnull(fp,',')				||
+	    skipnull(fp,',') /* me_parent */		||
+	    skipnull(fp,',') /* me_omen */		||
+	    skipnull(fp,',') /* me_toggle */		||
+	    skipnull(fp,',') /* me_select_val */	||
 	    skipwhite(fp,'}')				||
 	    skipwhite(fp,';'))
 		return(-1);
@@ -347,7 +349,25 @@ skipnull(fp,expected)
 FILE *fp;
 int expected;
 {
-	return(checkword(fp,"NULL") != 0 ? -1 : skipwhite(fp,','));
+	int ch;
+	/*return(checkword(fp,"NULL") != 0 ? -1 : skipwhite(fp,','));*/
+	switch ((ch = skipwhite(fp,ANY_CHAR) ))
+	{
+	case '0':
+	   ch =  skipwhite(fp,ANY_CHAR);
+	   break;
+	case 'N':
+	   ch =  checkword(fp,"ULL") != 0 ? -1 : skipwhite(fp,ANY_CHAR);
+	   break;
+	}
+
+	/* Allow any number of implicit NULL's or 0's at end of structure */
+	if ( ch == '}')
+	{
+            myungetc(ch,fp);
+	    ch = expected;
+	}
+	return (ch == expected ? 0 : -1);
 }
 
 static int
@@ -462,8 +482,30 @@ int expected;
 {
 	int ch;
 
-	while ((ch = mygetc(fp)) == ' ' || ch == '\t' || ch == '\n')
-		;
+	while ((ch = mygetc(fp)) == ' ' || ch == '\t' || ch == '\n' || ch == '/')
+	{
+	    if ( ch == '/')
+	    {
+	        int ch2 = mygetc(fp);
+	        if ( ch2 == '*')
+	        {
+		    while ((ch = mygetc(fp)) != EOF)
+		    {
+			if ( ch == '*')
+			{
+			   ch2 = mygetc(fp);
+			   if ( ch2 == '/')
+			      break;
+		           myungetc(ch2,fp);
+			}
+		    }
+	        } else
+	        {
+		    myungetc(ch2,fp);
+		    break;
+	        }
+	    }
+	}
 	if (ch == EOF) {
 		menerr = MBADFIL;
 		return(-1);
@@ -553,6 +595,91 @@ int expected;
 	*p_mnode = NULL;
 	return(skipwhite(fp,expected));
 }
+static int
+get_named_value(fp, p_value)
+FILE* fp;
+short* p_value;
+{
+    static struct named_value
+    {
+	const char* name;
+	unsigned short value;
+    } values[] = 
+    {
+	{ "ME_FONT", ME_FONT },
+	{ "ME_VER", ME_VER },
+	{ "ME_HOR", ME_HOR },
+	{ "ME_DIV", ME_DIV },
+	{ "ME_NSEL", ME_NSEL },
+	{ "ME_POPUP", ME_POPUP },
+	{ "ME_FREC", ME_FREC },
+	{ "ME_FREN", ME_FREN },
+	{ "ME_OPEN", ME_OPEN },
+	{ "ME_ISUB", ME_ISUB },
+	{ "ME_FREE", ME_FREE },
+	{ "ME_BGLINE", ME_BGLINE },
+	{ "ME_NOSCALE", ME_NOSCALE },
+	{ "ME_CREL", ME_CREL },
+	{ "ME_REDRAW", ME_REDRAW },
+	{ "ME_LEFT", ME_LEFT },
+    };
+    char name_buff[12];
+    int i = 0;
+    int ch = mygetc(fp);
+    int j;
+    while ((isalpha(ch) || ch == '_') && i < (sizeof(name_buff)-1))
+    {
+	name_buff[i++] = ch;
+	ch = mygetc(fp);
+    }
+    myungetc(ch, fp);
+    if ( i < (sizeof(name_buff)))
+    {
+	name_buff[i] = 0;
+	for ( j = 0; j < (sizeof(values)/sizeof(values[0])); j++)
+	{
+	    if ( !strcmp(name_buff, values[j].name ) )
+	    {
+		*p_value = values[j].value;
+		return 0;
+	    }
+	}
+    }
+
+    while(i)
+	myungetc(name_buff[--i],fp);
+    return -1;
+}
+
+static int
+getnumber(fp, p_res, base, minus)
+FILE *fp;
+short *p_res;
+int base;
+int minus;
+{
+    short res;
+    int ch;
+    for (res=0;;)
+    {
+	if ((ch = mygetc(fp)) == EOF)
+	{
+	    menerr = MBADFIL;
+	    return(-1);
+	}
+	if ((unsigned char)ch == ch && islower(ch))
+		ch = toupper(ch);
+	if (isdigit(ch) || (base == 16 && ch >= 'A' && ch <= 'F'))
+		res = res * base + ch - ((ch > '9') ? 'A'-10 : '0');
+	else
+	{
+	    *p_res = minus ? -res : res;
+	    myungetc(ch,fp);
+	    break;
+	}
+    }
+    return 0;
+}
 
 static int
 getnum(fp, p_res, followch)
@@ -560,11 +687,18 @@ FILE *fp;
 short *p_res;
 int followch;
 {
-	int ch, base, res, minus;
+   /* Get a short value, in decimal, hex, octal, as a character, or
+   ** as one of the flags recoginized by get_named_value().
+   ** Accept expressions envolving such values combined by '+'
+   */
+	short res;
+	int ret = 0;
+	int ch, base, minus;
 	
 	if ((ch = skipwhite(fp,ANY_CHAR)) == -1)
 		return(-1);
 	minus = 0;
+	base = 0;
 	if (ch == '0') {
 		if ((ch = mygetc(fp)) == 'x')
 			base = 16;
@@ -579,27 +713,49 @@ int followch;
 			minus = 1;
 		else
 			myungetc(ch,fp);
+	} else if ( isalpha(ch))
+	{
+	     myungetc(ch,fp);
+	     ret =  get_named_value(fp, &res);
 	}
-	else {
-		menerr = MBADFIL;
-		return(-1);
-	}
-	res = 0;
-	for (;;) {
-		if ((ch = mygetc(fp)) == EOF) {
-			menerr = MBADFIL;
-			return(-1);
+	else if (ch == '\'')
+	{
+	   res = mygetc(fp);
+	   if (res == '\\')
+	       res = mygetc(fp);
+	   if ( mygetc(fp) != '\'')
+	       ret = -1;
 		}
-		if ((unsigned char)ch == ch && islower(ch))
-			ch = toupper(ch);
-		if (isdigit(ch) || (base == 16 && ch >= 'A' && ch <= 'F'))
-			res = res * base + ch - ((ch > '9') ? 'A'-10 : '0');
 		else {
-			*p_res = minus ? -res : res;
-			myungetc(ch,fp);
-			return(skipwhite(fp,followch));
-		}
+		menerr = MBADFIL;
+	        ret = -1;
 	}
+	if (!ret &&  base )
+	{
+	    ret = getnumber(fp, &res, base, minus);
+	}
+	if (!ret)
+	{
+	    ch = skipwhite(fp,ANY_CHAR);
+	    if ( ch == '+' )
+	    {
+		short res2;
+		ret = getnum(fp, &res2, followch);
+		res += res2;
+	    } else if ( ch != followch)
+		ret = -1;
+	}
+	if (!ret)
+	   *p_res = res;
+	return ret;
+}
+
+static int
+getrv(fp, p_res, followch)
+FILE *fp;
+short *p_res;
+int followch;
+{
 }
 
 static int
@@ -709,3 +865,4 @@ FILE *fp;
 	if (c == '\n')
 		merr.mr_line--;
 }
+
