@@ -18,7 +18,9 @@ char Men3_wdbyte_c_sccsid[] = "@(#)wdbyte.c	1.10 22/12/93 (UKC)";
 #include "menu_priv.h"
 #include "wdbyte.h"
 
+#ifndef NULL
 #define NULL 0
+#endif
 
 /*  Emulation of the perq wdbyte(2) system call over wn.
  *  Deals correctly with variable width fonts, and the various
