@@ -7,7 +7,7 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char arg_version_sccsid[] = "@(#)arg_version.c	1.5 26/4/92 (UKC)";
+char arg_version_sccsid[] = "@(#)arg_version.c	1.6 16 Sep 1994 (UKC)";
 
 
 #include <local/ukcprog.h>
@@ -19,5 +19,5 @@ char arg_version_sccsid[] = "@(#)arg_version.c	1.5 26/4/92 (UKC)";
 const char *
 arg_version()
 {
-	return _arg_sccsdata[0];
+	return arg__sccsdata[0];
 }
