@@ -15,7 +15,8 @@ Text_buffer *text_create_lbuf_buffer PROTO((alloc_pool_t *ap,
 					    void *handle,
 					    Text_info_func info_func,
 					    Text_read_func read_func,
-					    Text_close_func close_func));
+					    Text_close_func close_func,
+					    bool apool_is_scratch));
 #endif
 
 #ifndef TEXT_BUFFER_DEFINED
