@@ -229,7 +229,7 @@ Edit_propchange *pc;
 	for (pc2 = pl->pclist; pc2 != pc; pc2 = pc2->next) {
 		if (pc2 == NULL)
 /*	RGA		panic("prop not found in edp");*/
-			errf("prop not found in edp");
+			break;	/* RGA can happen when running with changed target */
 		last = pc2;
 	}
 
