@@ -781,8 +781,17 @@ Edit_propchange **p_pc;
 				need_render_block = pc != NULL;
 			}
 		}
-		else if (*iptr == '\n' && rc->stop_at_newline) {
-			break;
+		else if (   rc->stop_at_newline
+			 && ( *iptr == '\n'
+			     || (*iptr == '\r' && iptr[1] == '\n')))
+		{
+		    /* RCB: For files with DOS end-of-line records */
+		    if ( *iptr == '\r')
+		    {
+			point++;
+			iptr++;
+		    }
+		    break;
 		}
 		else {
 			const char *charstr, *sptr, *slim;
@@ -853,7 +862,9 @@ Edit_propchange **p_pc;
 		edit_get_bytes(d->buffer, point, &iptr, &ilim);
 
 	line_continues = iptr != ilim &&
-		         !(*iptr == '\n' && rc->stop_at_newline);
+		         !(   rc->stop_at_newline
+			       && (   *iptr == '\n'
+			           || (*iptr == '\r' && iptr[1] == '\n')));
 	if (line_continues && !rc->fold_lines)
 	{
 	  int char_width;
