@@ -9,9 +9,9 @@
 
 char utils_memxxx_sccsid[] = "@(#)memxxx.c	1.6 26/4/92 (UKC)";
 
+#include <ifdefs.h>
 #include <stdlib.h>
 
-#include "ifdefs.h"
 
 #ifdef ARCH_CLIPPER
 #define NO_MEMXXX
