@@ -23,7 +23,6 @@ char wn_wn_cu_c_sccsid[] = "@(#)wn_cu.c	1.14 1/3/93 (UKC)";
 #ifdef X11
 #define NULL_CID	0
 
-static Pixmap bm_to_pixmap PROTO((bitmap_t *bm));
 static void toggle_shorts PROTO((unsigned short *s, int count));
 #endif /* X11 */
 
@@ -41,7 +40,7 @@ XColor *fg_color, *bg_color;
 		Cu_bg_color = *bg_color;
 }
 
-static Pixmap
+Pixmap
 bm_to_pixmap(bm)
 bitmap_t *bm;
 {
@@ -218,6 +217,19 @@ cursor_t cid;
 #endif /* SUNVIEW */
 #ifdef X11
 	XDefineCursor(wn__Dpy, w->w_win, (unsigned)cid);
+
+	/* UPS split windows. */
+	if ((wn == WN_STDWIN) && (wn_NRootwin > 1 ))
+	{
+	     int i;
+	     swin_t *w2;
+	     for (i = 1; i < wn_NRootwin; i++)
+	     {
+		  w2 = wn_RootWintab[i];
+	    XDefineCursor(wn__Dpy, w2->w_win, (unsigned)cid);
+	     }
+	}
+
 	wn__do_xflush();
 #endif /* X11 */
 	w->w_bw->bw_cid = cid;
