@@ -517,7 +517,7 @@ int *p_xyset;
 	int old_width, old_height, xbuttons, nbytes;
 	char ch;
 	char *keystr;
-	register swin_t *w, *w1, *w2;
+	register swin_t *w;
 	extern int _wn_Autoraise;
 
 	if (wn == WN_ANY) {
@@ -996,7 +996,7 @@ event_t *ev;
 
 		if (ev->ev_type == EV_WINDOW_RESIZED)
 			bw->bw_winch_event = EV_WINDOW_RESIZED;
-		else if (ev->ev_type == EV_WINDOW_EXPOSED &&
+		else if ((ev->ev_type == EV_WINDOW_EXPOSED) &&
 			 bw->bw_winch_event == 0)
 			bw->bw_winch_event = EV_WINDOW_EXPOSED;
 		else if (ev->ev_type == EV_LOST_SELECTION)
@@ -1290,7 +1290,6 @@ wn_expose_or_press_queued(wn, md, p_x, p_y, exposed, resized)
   swin_t *w;
   event_t event;
   int x, y, xyset, ret = 0;
-  unsigned long win;
 
   if (_wn_Windowtab)
   {
