@@ -47,6 +47,7 @@ char ups_ao_elflib_c_sccsid[] = "@(#)ao_elflib.c	1.1 24/5/95 (UKC)";
 #include "data.h"		/* RGA for dread & dgets */
 #include "state.h"
 #include "obj_bpt.h"
+#include "va.h"
 
 struct Solib_addr {
 	dev_t dev;
@@ -59,6 +60,7 @@ struct Solib_addr {
 	off_t offset;
 	taddr_t vaddr;
 	size_t pagesize;
+  	char *path;		/* RGA for unloaded libs */
 	Solib_addr *next;
 };
 
@@ -177,7 +179,6 @@ make_libdep(alloc_pool_t *ap, const char *name, Libdep *par, long mtime)
 	ld->emitted_noload_msg = FALSE;
 	ld->no_reload = FALSE;
 	ld->wants_load = user_wants_library_loaded((char *)name);
-	
 	return ld;
 }
 
@@ -270,11 +271,12 @@ load_elf_shlib(target_t *xp, Libdep *ld)
 	long mtime = 0;
 	struct stat stbuf;
 	static int verbose = -1;
+	Solib_addr *sa;
 
 	ip = GET_IPROC(xp);
 
 	if (ld->no_reload == TRUE)
-	  return TRUE;
+	  return FALSE;		/* RGA change from TRUE to force scan of next lib */
   
 	if (verbose == -1)
 	{
@@ -322,57 +324,86 @@ load_elf_shlib(target_t *xp, Libdep *ld)
 
 	if (ld->wants_load == FALSE)
 	{
+	  add_to_noload_list(path, 0);
 	  if (verbose && get_message_wn() == -1)
 	    if (ld->emitted_noload_msg == FALSE)
 	    {
 	      ld->emitted_noload_msg = TRUE;
 	      fprintf(stderr, "NOT loading symtab %s\n", path);
 	    }
-	  ok = FALSE;
 	}
-	else
+
+	ok = (open_for_reading(path, "shared library", &fd) &&
+	      check_elf_file_type(fd, path, ET_DYN, "load symbols from"));
+	if (ok)
 	{
-	  ok = (open_for_reading(path, "shared library", &fd) &&
-		check_elf_file_type(fd, path, ET_DYN, "load symbols from"));
-	  if (ok)
+	  if (fstat(fd, &stbuf) == 0)
+	    mtime = stbuf.st_mtime;
+	  if (ld->wants_load == FALSE)
+	  {
+	    /*	    ok = FALSE;*/
+	    for (sa = ip->ip_solib_addrs; sa != NULL; sa = sa->next) {
+	      if (sa->dev == stbuf.st_dev && sa->ino == stbuf.st_ino)
+		break;
+	    }
+
+#ifdef OS_SUNOS
+	    /* RGA if solib was a mvfs file (e.g. Rational Clearcase), */
+	    /* procfs file may point to a nfs equivalent, so use the */
+	    /* st_size, mtv_sec and mtv_nsec fields to match */
+
+	    if (sa == NULL)
+	      for (sa = ip->ip_solib_addrs; sa != NULL; sa = sa->next) {
+		if (stbuf.st_fstype && !strcmp(stbuf.st_fstype, "mvfs") && 
+		    sa->size == stbuf.st_size &&
+		    sa->mtv_sec == stbuf.st_mtim.tv_sec &&
+		    sa->mtv_nsec == stbuf.st_mtim.tv_nsec)
+		  break;
+	      }
+#endif	    
+	    if (sa != NULL)
+	    {
+	      if (sa->path)
+		  free(sa->path);
+	      sa->path = path;
+	    }
+	  }
+	  else
 	  {
 	    int wn = get_message_wn();
 
-	    if (fstat(fd, &stbuf) == 0)
-	      mtime = stbuf.st_mtime;
+	    add_to_noload_list(path, 1); /* mark as loaded */
 	    if (wn != -1)
 	    {
-	      wn_updating_off(wn);
 	      errf("\b%s symtab %s...", reload ? "Reloading" :
 		   "Loading", path);
-	      wn_updating_on(wn);
+	      wn_do_flush();
 	    }
 	    if (verbose == 1)
 	      fprintf(stderr, "%s symtab %s...", reload ? "Reloading" :
 		      "Loading", path);
 	    ok = scan_elf_symtab(xp->xp_apool, path, fd, ld, (func_t **)NULL,
-				 (func_t **)NULL, &ip->ip_solibs);
+				 (func_t **)NULL, &ip->ip_solibs,
+				 xp->xp_target_updated);
 	    if (xp->xp_hit_solib_event == TRUE)
 	      xp->xp_new_dynamic_libs_loaded = TRUE;
 	    if (wn != -1)
 	    {
-	      wn_updating_off(wn);
 	      errf("\b%s symtab %s...done", reload ? "Reloading" :
-		     "Loading", path);
-	      wn_updating_on(wn);
+		   "Loading", path);
+	      wn_do_flush();
 	    }
 	    if (verbose == 1)
-		fputs("done\n", stderr);
+	      fputs("done\n", stderr);
 	  }
 	}
-
-	free(path);
-
-	if (ok) {
-		Solib_addr *sa;
-		
+	
+	if (ld->wants_load == TRUE || sa == NULL)
+	  free(path);
+	
+	if (ok && ld->wants_load) {
+	  	ld->no_reload = FALSE;
 		so = ld->so;
-		
 		so->symtab->st_modtime = stbuf.st_mtime;
 
 		/*  TODO: should pass load address to scan_elf_symtab().
@@ -421,18 +452,34 @@ target_t *xp;
 	
 	for (sa = ip->ip_solib_addrs; sa != NULL; sa = next) {
 		next = sa->next;
-		free(sa);
+		if (sa->path)
+		  free(sa->path);
+		/* RGA Purify gives a FUM here */
+		/* free(sa);*/
 	}
 
 	ip->ip_solib_addrs = NULL;
 }
 
+static int
+libdep_find_name(new_libdep, name)
+     Libdep *new_libdep;
+     const char *name;
+{
+  Libdep *ld;
+
+  for (ld = new_libdep; ld; ld = ld->next)
+    if (!strcmp(ld->name_only, name))
+      return 1;			/* found */
+  return 0;
+}
+
 bool
 elf_get_dynamic_solibs(alloc_pool_t *target_ap, const char *path,
-		       Solib **p_solibs, bool check_only)
+		       Solib **p_solibs, bool check_only, bool save_state)
 {
     Libdep *ld, first_child, *last_child, *ld1, *ld2;
-    int same;
+    int same, same_name, times_differ;
 
     if (Main_debug_vaddr != 0)
     {
@@ -442,27 +489,81 @@ elf_get_dynamic_solibs(alloc_pool_t *target_ap, const char *path,
 				    &last_child);
 	last_child->next = NULL;
 	ld->deps = first_child.next;
+
 	for (same = 1, ld1 = (*p_solibs)->latest_libdep->deps, ld2 = ld->deps;
 		 ld1 && ld2; ld1 = ld1->next, ld2 = ld2->next)
-	    if ((strcmp(ld1->name_only, ld2->name_only)  && ld2->wants_load == TRUE) ||
-		  (ld1->wants_load == FALSE && ld2->wants_load == TRUE))
-	    {
-		/* RGA was noload before, now is not, after init file rescan */
-		same = 0;
-		break;
-	    }
+	{
+	  same_name = !strcmp(ld1->name_only, ld2->name_only);
+	  times_differ = same_name && ld1->mtime != ld2->mtime;
+	  if (times_differ ||
+	      (!same_name  && ld2->wants_load == TRUE) ||
+	      (ld1->wants_load == FALSE && ld2->wants_load == TRUE))
+	  {
+	    /* RGA was noload before, now is not, after init file rescan */
+	    /* RGA replaced by a newer lib of diff name, so deactivate old */
+	    /* or library has been rebuilt with no name change. */
+	    same = 0;
+	    break;
+	  }
+	}
 	if (same == 0 || ld2) /* New list is longer */
 	{
-	    if (!check_only)
-		(*p_solibs)->latest_libdep->deps = ld->deps; /* RGA is this right? */
-	    return TRUE;
+	  target_t *xp;
+
+	  /* RGA if old libdep is not in the new list, mark the st as eclipsed */
+	  for (ld1 = (*p_solibs)->latest_libdep->deps; ld1; ld1 = ld1->next)
+	    if (!libdep_find_name(ld->deps, ld1->name_only) || times_differ)
+	    {
+	      ld1->no_reload = 1;
+	      if ( ld1->so )
+		ld1->so->symtab->st_eclipsed = 1;
+	    }
+	  xp = get_current_target();
+	  if (save_state == TRUE)
+	    save_target_state(xp, TRUE, TRUE, FALSE);
+	  
+	  if (check_only == FALSE)
+	  {
+	    (*p_solibs)->latest_libdep->deps = ld->deps; /* RGA is this right? */
+	    load_from_solibs(xp, 0); /* load all libs that user wants */
+	  }
+	  
+	  if (save_state == TRUE)
+	    restore_target_state(FALSE, TRUE);
+	  return times_differ ? TRUE : FALSE;
 	}
     }
     return FALSE;
 }
 
+void
+elf_update_dynamic_solibs(alloc_pool_t *target_ap, const char *path,
+		       Solib **p_solibs, bool load_libs)
+{
+    Libdep *ld, first_child, *last_child;
+
+    if (Main_debug_vaddr != 0)
+    {
+	ld = make_libdep(target_ap, path, (Libdep *)NULL, 0);
+	last_child = &first_child;
+	elf_get_dynamic_shlib_info (target_ap, ld, path, Main_debug_vaddr,
+				    &last_child);
+	last_child->next = NULL;
+	ld->deps = first_child.next;
+	(*p_solibs)->latest_libdep->deps = ld->deps; /* RGA is this right? */
+	if (load_libs)
+	{
+	  target_t *xp;
+
+	  xp = get_current_target();
+	  load_from_solibs(xp, 0); /* load all libs that user wants */
+	}
+    }
+    return;
+}
+
 bool
-elf_note_shlib_addr(iproc_t *ip, dev_t dev, ino_t ino, 
+elf_note_shlib_addr(alloc_pool_t *ap, iproc_t *ip, dev_t dev, ino_t ino, 
 		    off_t size, long mtv_sec,  long mtv_nsec, 
 		    off_t offset, taddr_t vaddr, size_t pagesize)
 {
@@ -491,7 +592,7 @@ elf_note_shlib_addr(iproc_t *ip, dev_t dev, ino_t ino,
 	else {
 		Solib_addr *sa;
 		
-		sa = e_malloc(sizeof(Solib_addr));
+		sa = alloc(ap, sizeof(Solib_addr));
 		sa->dev = dev;
 		sa->ino = ino;
 		sa->size = size;
@@ -502,6 +603,7 @@ elf_note_shlib_addr(iproc_t *ip, dev_t dev, ino_t ino,
 		sa->offset = offset;
 		sa->vaddr = vaddr;
 		sa->pagesize = pagesize;
+		sa->path = NULL;
 		sa->next = ip->ip_solib_addrs;
 		ip->ip_solib_addrs = sa;
 	}
@@ -570,7 +672,7 @@ elf_get_core_shlib_info(alloc_pool_t *ap, iproc_t *ip)
 			continue;
 		}
 
-		elf_note_shlib_addr(ip, stbuf.st_dev, stbuf.st_ino,
+		elf_note_shlib_addr(ap, ip, stbuf.st_dev, stbuf.st_ino,
 				    0, 0, 0,
 				    0, (taddr_t)lmap.l_addr, 0);
 		/* Here libpath is a full path, when we first got the
@@ -699,7 +801,7 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 		(*last_child) = (*last_child)->next;
 
 #ifdef AO_USE_PTRACE
-		elf_note_shlib_addr(ip,
+		elf_note_shlib_addr(ap, ip,
 				    stbuf.st_dev, stbuf.st_ino,
 				    stbuf.st_size,
 				    0, 0,
@@ -712,15 +814,29 @@ elf_get_dynamic_shlib_info(alloc_pool_t *ap, Libdep *ld, const char *textpath,
 bool
 scan_main_elf_symtab(alloc_pool_t *target_ap, const char *path, int fd,
 		     long modtime, Solib **p_solibs, Solib_addr **p_solib_addrs,
-		     struct func_s **p_mainfunc, struct func_s **p_initfunc)
+		     struct func_s **p_mainfunc, struct func_s **p_initfunc,
+		     bool target_updated)
 {
 	Libdep *ld;
 	bool ok;
 
+	Main_debug_vaddr = 0;	/* RGA for reattach with rescan */
 	*p_solibs = NULL;
 	ld = make_libdep(target_ap, path, (Libdep *)NULL, 0);
+	if (target_updated == TRUE)
+	{
+	  errf("\bScanning symbols of `%s'...", path);
+	  wn_do_flush();
+	}
 	ok = scan_elf_symtab(target_ap, path, fd, ld,
-			       p_mainfunc, p_initfunc, p_solibs);
+			     p_mainfunc, p_initfunc, p_solibs,
+			     target_updated);
+	if (target_updated == TRUE)
+	{
+	  errf("\bScanning symbols of `%s'... done", path);
+	  wn_do_flush();
+	}
+
 	if (ld->so)
 	{
 	  ld->so->symtab->st_modtime = modtime;
@@ -786,31 +902,37 @@ next_ld(Libdep *cur)
 bool
 elf_next_symtab(target_t *xp, symtab_t *st, bool load_new, symtab_t **p_next_st)
 {
-	Solib *so;
+	Solib *so, *so1;
 	Libdep *ld;
 	
-	if (st == NULL) {
-		*p_next_st = GET_IPROC(xp)->ip_solibs->symtab;
-
-		if (!(*p_next_st)->st_eclipsed)
-		  return TRUE;
-	}
-
+	if (st == NULL) 
+	  for (so = GET_IPROC(xp)->ip_solibs; so != NULL; so = so->next) {
+	    if (!so->symtab->st_eclipsed) {	 
+	      *p_next_st = so->symtab;
+	      return TRUE;
+	    }
+	  }
+	
 	so = AO_STDATA(st)->st_solib;
 
 	/* RGA 07/18/00 add a check so the extensive lookup is done
-	   only when the dynamic linker breakpoint has been set */
+	   only when the dynamic linker breakpoint has been set.
+
+	   RGA 01/30/01 Older library versions are marked
+	   with st_eclipsed, so skip those. */
+
 	if (xp->xp_hit_solib_event == FALSE)
 	{
-	  if (so->next != NULL) {	 
-	    *p_next_st = so->next->symtab;
-	    return TRUE;
+	  for (so1 = so->next; so1 != NULL; so1 = so1->next) {
+	    if (!so1->symtab->st_eclipsed) {	 
+	      *p_next_st = so1->symtab;
+	      return TRUE;
+	    }
 	  }
 	  
 	  if (!load_new || so->is_last_symtab) 
 	    return FALSE;
 	}
-
 	for (ld = next_ld(so->latest_libdep); ld != NULL; ld = next_ld(ld)) {
 		if (!load_elf_shlib(xp, ld))
 			continue;
@@ -996,37 +1118,6 @@ add_solib_entry(alloc_pool_t *ap, symtab_t *st, Elfinfo *el, Solib **p_solibs)
 	return TRUE;
 }
 
-int
-elf_save_symtab_breakpoints(xp, remove_bpts)
-target_t *xp;
-bool remove_bpts;
-{
-	FILE *fp;
-	iproc_t *ip;
-	
-	ip = GET_IPROC(xp);
-	if (!ip->ip_solibs)
-	  return 0;
-
-	if ((fp = fopen(get_temp_bpt_filename(), "w")) == NULL)
-	{
-	  errf("Can't create temp saved breakpoints file `%s'",
-	       get_temp_bpt_filename());
-	  return 0;
-	}
-	save_all_breakpoints((char *)fp);
-	if (remove_bpts == TRUE)
-	  remove_all_breakpoints();
-	fclose(fp);
-	return 1;
-}
-
-void
-elf_restore_symtab_breakpoints()
-{
-  restore_cached_bpts();
-}
-
 void
 unload_shared_library_symtabs(xp)
 target_t *xp;
@@ -1035,8 +1126,9 @@ target_t *xp;
 }
 
 void
-elf_rescan_dynamic_solibs(xp)
+elf_rescan_dynamic_solibs(xp, check_only)
 target_t *xp;
+bool check_only;
 {
 #ifdef AO_ELF
   iproc_t *ip;
@@ -1045,7 +1137,7 @@ target_t *xp;
   if (ip->ip_pid)
     xp->xp_new_dynamic_libs = 
       elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-			     &ip->ip_solibs, FALSE);
+			     &ip->ip_solibs, check_only, TRUE); /* save state */
 #endif
 }
 
@@ -1148,6 +1240,7 @@ Solib_addr **p_solib_addrs;
 	    sa->offset = 0;
 	    sa->vaddr = addr;
 	    sa->pagesize = 0;
+	    sa->path = NULL;
 	    sa->next = *p_solib_addrs;
 	    *p_solib_addrs = sa;
 	}
@@ -1169,13 +1262,16 @@ static const char **
 add_to_env(s)
 const char *s;
 {
+#if !HAVE_DEFINED_ENVIRON
 	extern const char **environ;
+#endif
 	const char **sptr, **envp, **src, **dst;
 
-	for (sptr = environ; *sptr != NULL; ++sptr)
+	for (sptr = (const char **)environ; *sptr != NULL; ++sptr)
 		;
-	envp = dst = (const char **)e_malloc(sizeof(char *) * (sptr - environ + 2));
-	src = environ;
+	envp = dst = (const char **)e_malloc(sizeof(char *) *
+					     (sptr - (const char **)environ + 2));
+	src = (const char **)environ;
 	while (*src != NULL)
 		*dst++ = *src++;
 	*dst++ = s;
@@ -1184,4 +1280,161 @@ const char *s;
 }
 #endif
 
+
+typedef struct addr_s{
+  taddr_t addr;
+  const char *path;
+} addr_t;
+
+static int
+addr_cmp(addr1, addr2)
+addr_t *addr1, *addr2;
+{
+  if (!addr1 || !addr2)
+    return 1;
+  return addr1->addr < addr2->addr;
+}
+
+char *
+get_unloaded_lib(addr)
+taddr_t addr;
+{
+  iproc_t *ip;
+  Solib_addr *sa;
+  int i;
+  addr_t *n, *nroot;
+  char *ret;
+  
+  ip = GET_IPROC(get_current_target());
+  for (i = 0, sa = ip->ip_solib_addrs; sa != NULL; sa = sa->next)
+    if (sa->vaddr)
+      i++;
+  if (i == 0)
+    return 0;
+  nroot = n = malloc(sizeof(addr_t)*(i+1));
+  for (sa = ip->ip_solib_addrs; sa != NULL; sa = sa->next, n++)
+    if (sa->vaddr)
+    {
+      n->addr = sa->vaddr;
+      n->path = sa->path;
+    }
+  n->addr = 0;
+  n->path = 0;
+  qsort((char *)nroot, i, sizeof(addr_t), addr_cmp);
+
+  for (n = nroot; n->addr; n++)
+    if (addr > n->addr)
+    {
+      ret = (char *)n->path;
+      free(nroot);
+      return ret;
+    }
+  free(nroot);
+  return 0;
+}
+
+void
+reset_main_debug_address()
+{
+	Main_debug_vaddr = 0;	/* RGA for reattach with rescan */
+}
+
+static void
+load_from_libdep(xp, par, level, name)
+     target_t *xp;
+     Libdep *par; 
+     int level;
+     char *name;
+{
+  Libdep *ld;
+	
+  if (!name && par->par)
+    load_elf_shlib(xp, par);
+  else 
+    if (par->par && !strcmp(par->name, name))
+    {
+      load_elf_shlib(xp, par);
+      return;
+    }
+  
+  for (ld = par->deps; ld != NULL; ld = ld->next)
+    load_from_libdep(xp, ld, level + 1, name);
+}
+
+void
+load_from_solibs(xp, name)
+     target_t *xp;
+     char *name;
+{
+  Solib *so;
+  iproc_t *ip;
+
+  ip = GET_IPROC(xp);
+
+  for (so = ip->ip_solibs; so != NULL; so = so->next)
+  {
+    if (!name && so->latest_libdep->par)
+      load_elf_shlib(xp, so->latest_libdep);
+    else 
+      if (so->latest_libdep->par && !strcmp(so->symtab->st_path, name))
+      {
+	load_elf_shlib(xp, so->latest_libdep);
+	return;
+      }
+  }
+  load_from_libdep(xp, ip->ip_solibs->latest_libdep, 0, name);
+}
+
+/* RGA a hack to attempt to reuse previously read libraries when attaching or
+   starting a target which is newer or of a different name. Need to change
+   procfs_attach_to_process() too for this to work */
+void
+elf_make_main_symtab(xp, target_updated, pid)
+target_t *xp;
+bool target_updated;
+int pid;
+{
+  Libdep *ld;
+  int fd, i;
+  long mtime;
+  iproc_t *ip;
+
+  ip = GET_IPROC(xp);
+  
+#if AO_USE_PROCFS
+  procfs_init(ip);
+#else
+	ip->ip_ptrace_info = NULL;
+#if defined(AO_AOUT_SHLIBS)
+	ip->ip_shlibs_ap = NULL;
+	ip->ip_shlibs = NULL;
+#endif
+#endif
+  ld = make_libdep(xp->xp_apool, xp->xp_textpath, (Libdep *)NULL, 0);
+  if ((check_target(xp->xp_textpath, &fd, &i, &mtime, FALSE)) == -1)
+    return;
+  if (target_updated == TRUE)
+  {
+    errf("\bScanning symbols of `%s'...", xp->xp_textpath);
+    wn_do_flush();
+  }
+  scan_elf_symtab(xp->xp_apool, xp->xp_textpath, fd, ld, &xp->xp_mainfunc,
+		  &xp->xp_initfunc, &ip->ip_solibs, target_updated);
+  if (target_updated == TRUE)
+  {
+    errf("\bScanning symbols of `%s'...done", xp->xp_textpath);
+    wn_do_flush();
+  }
+  if (ld->so)
+  {
+    ld->so->symtab->st_modtime = mtime;
+#ifdef AO_USE_PTRACE /* interp */
+    if (!get_preload_shlib_list (xp->xp_apool, xp->xp_textpath, &ip->ip_solibs,
+				 &ip->ip_solib_addrs))
+      return;
+#endif
+    Main_debug_vaddr = ld->so->debug_vaddr;
+  }
+}
+
 #endif /* AO_ELF */
