@@ -28,14 +28,16 @@ void elf_zap_solib_addrs PROTO((target_t *xp));
 
 void dump_elf_libs PROTO((target_t *xp));
 
-int elf_save_symtab_breakpoints PROTO((target_t *xp, bool remove_bpts));
+void elf_rescan_dynamic_solibs PROTO((target_t *xp, bool check_only));
 
-void elf_rescan_dynamic_solibs PROTO((target_t *xp));
+void load_from_solibs PROTO((target_t *xp, char *name));
+
+void elf_make_main_symtab PROTO((target_t *xp, bool target_updated, int pid));
 
 #endif
 
 #ifdef AO_TARGET_H_INCLUDED
-bool elf_note_shlib_addr PROTO((iproc_t *ip, dev_t dev, ino_t ino,
+bool elf_note_shlib_addr PROTO((alloc_pool_t *ap, iproc_t *ip, dev_t dev, ino_t ino,
 				off_t size, long mtv_sec,  long mtv_nsec,
 				off_t offset, taddr_t vaddr, size_t pagesize));
 
@@ -44,18 +46,22 @@ bool elf_get_core_shlib_info PROTO((alloc_pool_t *ap, iproc_t *ip));
 
 struct func_s;
 bool elf_get_dynamic_solibs PROTO((alloc_pool_t *target_ap, const char *path,
-				   Solib **p_solib, bool check_only));
+				   Solib **p_solib, bool check_only, bool save_state));
+
+void elf_update_dynamic_solibs PROTO((alloc_pool_t *target_ap, const char *path,
+				   Solib **p_solib, bool load_libs));
 
 bool scan_main_elf_symtab PROTO((alloc_pool_t *target_ap, const char *path,
 				 int fd, long modtime, Solib **p_solib,
 				 Solib_addr **p_solib_addrs,
 				 struct func_s **p_mainfunc,
-				 struct func_s **p_initfunc));
-
-void elf_restore_symtab_breakpoints PROTO((void));
+				 struct func_s **p_initfunc,
+				 bool target_updated));
 
 #ifdef SYMTAB_H_INCLUDED
 bool add_solib_entry PROTO((alloc_pool_t *ap, symtab_t *st, Elfinfo *el,
 			    Solib **p_solibs));
 #endif
 
+void reset_main_debug_address PROTO((void));
+char *get_unloaded_lib PROTO((taddr_t addr));
