@@ -54,6 +54,7 @@ char ups_ao_elfsym_c_sccsid[] = "@(#)ao_elfsym.c	1.3 20 Jun 1995 (UKC) [Patched]
 #include "elfstab.h"
 #include "srcpath.h"
 #include "util.h"
+#include "va.h"
 
 static bool Had_error;
 	
@@ -830,7 +831,8 @@ set_function_addresses(Elfinfo *el, symtab_t *st, func_t **p_flist)
 	fil_ht = make_fil_hash(hash_ap, st->st_sfiles);
 	
 	hv = hash_make_hashvalues();
-	
+	st->st_func_hv = (char *)hv;
+
 	for (i = 0; i < nsyms; ++i) {
 		Elf32_Sym *sym;
 		const char *name;
@@ -922,7 +924,7 @@ set_function_addresses(Elfinfo *el, symtab_t *st, func_t **p_flist)
  */
 bool
 scan_elf_symtab(target_ap, textpath, fd, libdep, p_mainfunc,
-		p_initfunc, p_solibs)
+		p_initfunc, p_solibs, reattach_with_rescan)
 alloc_pool_t *target_ap;
 const char *textpath;
 int fd;
@@ -930,6 +932,7 @@ Libdep *libdep;
 func_t **p_mainfunc;
 func_t **p_initfunc;
 Solib **p_solibs;
+bool reattach_with_rescan;
 {
 	symtab_t *st;
 	Execinfo eibuf;
@@ -964,7 +967,6 @@ Solib **p_solibs;
 	initfunc_name = "_init"; /* RGA temp */
 	do_ao_postscan_stuff(st, flist, &eibuf, p_mainfunc, mainfunc_name,
 			     p_initfunc, initfunc_name);
-
 	return TRUE;
 }
 
