@@ -26,5 +26,6 @@ bool scan_elf_symtab PROTO((alloc_pool_t *target_ap,
 			    Libdep *libdep,
 			    struct func_s **p_mainfunc,
 			    struct func_s **p_initfunc,
-			    Solib **p_solibs));
+			    Solib **p_solibs,
+			    bool target_updated));
 #endif
