@@ -142,7 +142,9 @@ close_procfs(Procfs_info *pi)
 {
 	if (pi->procfd != -1)
 		close(pi->procfd);
-	
+
+	if (pi->handlers)
+	  free(pi->handlers);
 	free(pi);
 }
 
@@ -171,8 +173,10 @@ setup_procfs_in_parent(iproc_t *ip, int pid, bool attached)
 	memset((char *)pi, '\0', sizeof(Procfs_info));
 
 	if (!open_procfs_file(pid, pi->path, attached, &pi->procfd)) {
-		free(pi);
-		return FALSE;
+	  if (pi->handlers)
+	    free(pi->handlers);
+	  free(pi);
+	  return FALSE;
 	}
 
 	if (!procfn(pi, PIOCMAXSIG, &pi->maxsig, "do PIOCMAXSIG") != 0)
@@ -280,7 +284,7 @@ note_target_dead(target_t *xp)
 
 	mark_breakpoints_as_uninstalled(xp);
 
-/*	elf_zap_solib_addrs(xp);  RGA temp */
+	elf_zap_solib_addrs(xp);
 }
 
 static bool
@@ -611,7 +615,7 @@ procfs_load_mmap_info(target_t *xp, bool attached)
 				seen->next = seenlist;
 				seenlist = seen;
 
-				elf_note_shlib_addr(GET_IPROC(xp),
+				elf_note_shlib_addr(xp->xp_apool, GET_IPROC(xp),
 						    stbuf.st_dev, stbuf.st_ino,
 						    stbuf.st_size,
 #ifndef OS_SUNOS
@@ -726,51 +730,20 @@ stopres_t *p_stopres;
 	}
 
 	*p_stopres = xp_execto(xp, main_addr);
-	rescan_dynamic_solibs(xp);
 	if (*p_stopres != SR_BPT) {
 		errf("Can't run process to start of main function");
 		if (*p_stopres != SR_DIED)
 			xp_kill(xp);
 		return -1;
 	}
-	/* RGA This block was commented out at some post 3.33 point,
-	   presumably to improve speed, but it is necessary in
-	   order to detect loading of dynamic libs. It has no
-	   effect unless we have hit the dynamic linker breakpoint */
-#ifdef AO_ELF
-	if (*p_stopres == SR_BPT)
-	{
-	  if (xp->xp_hit_solib_event == TRUE)
-	    elf_save_symtab_breakpoints(xp, FALSE);
-	  while (xp->xp_hit_solib_event == TRUE)
-	  {
-	    rescan_dynamic_solibs(xp);
-	    refresh_target_display(xp, *p_stopres, FALSE);
-	    if (xp->xp_new_dynamic_libs_loaded == TRUE &&
-		can_install_all_breakpoints(xp) == TRUE)
-	       {
-		 remove_all_breakpoints();
-		 procfs_load_mmap_info(xp, FALSE);
-		 redo_displayed_source_files();
-		 elf_restore_symtab_breakpoints();
-		 xp->xp_new_dynamic_libs_loaded = FALSE;
-		 xp->xp_hit_solib_event = FALSE;
-	       }
-	    *p_stopres = xp_cont(xp);
-	    if (*p_stopres != SR_BPT ||
-		user_wants_stop(TRUE)) /* RGA peek in queue */
-	    {
-	      xp->xp_new_dynamic_libs_loaded = FALSE;
-	      xp->xp_hit_solib_event = FALSE;
-	      break;
-	    }
-	  }
-	}
-#endif
 	if (!procfs_load_mmap_info(xp, FALSE)) {
 		xp_kill(xp);
 		return -1;
 	}
+#ifdef AO_ELF
+	elf_update_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
+				  &ip->ip_solibs, TRUE); 
+#endif
 
 	return 0;
 }
@@ -803,42 +776,24 @@ int pid;
 	wait_for_target_to_stop(xp);
 	
 	reattach = Last_attach_pid != 0;
-	if (reattach)
-	{
-	  free_library_load_list();
-	  if (td_init_from())
-	  {
-	    bool eof;
-	    int res;
-	    
-	    res = td_load_loop(&eof);
-	    
-	    if (res != 0)
-	      fprintf(stderr, "Error in init file\n");
-	    td_restore_replay_fp();
-	  }
-	}
 
-#ifdef AO_ELF
-	/* RGA for dynamic libs */
-	{
-	  set_current_target(xp);
-	  if (initial_last_attach_pid != 0 &&
-	      xp->xp_restore_bpts == FALSE) /* not first attach */
-	  {
-	    elf_save_symtab_breakpoints(xp, TRUE);
-	    xp->xp_restore_bpts = TRUE;
-	  }
-	  xp->xp_new_dynamic_libs = 
-	    elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-				   &ip->ip_solibs, FALSE);
-	}
-#endif
 	if (!procfs_load_mmap_info(xp, TRUE)) {
-		close_procfs(ip->ip_procfs_info);
-		ip->ip_procfs_info = NULL;
-		return -1;
+	  close_procfs(ip->ip_procfs_info);
+	  ip->ip_procfs_info = NULL;
+	  return -1;
 	}
+	if (reattach)
+	  rescan_init_file();
+
+	set_current_target(xp);
+	/* RGA this clause needs to skipped when using the
+	   incomplete elf_make_main_symtab() hack */
+#ifdef AO_ELF
+	xp->xp_new_dynamic_libs = 
+	  elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
+				 &ip->ip_solibs, TRUE, FALSE);
+	elf_update_dynamic_solibs(xp->xp_apool, xp->xp_textpath, &ip->ip_solibs, TRUE);
+#endif
 	return 0;
 }
 
@@ -1286,12 +1241,12 @@ size_t nbytes;
 }
 
 int
-procfs_write_corefile(ip, name)
-iproc_t *ip;
-const char *name;
+procfs_load_library(xp, arg)
+target_t *xp;
+char *arg;
 {
-	errf("procfs_write_corefile NYI");
-	return -1;
+  load_library_input((void *)xp, arg);
+  return -1;
 }
 
 #endif /* AO_USE_PROCFS */
