@@ -26,7 +26,7 @@ int procfs_read_data PROTO((iproc_t *ip, taddr_t addr, char *buf,
                             size_t nbytes));
 int procfs_write_data PROTO((iproc_t *ip, taddr_t addr, const char *buf, 
                              size_t nbytes));
-int procfs_write_corefile PROTO((iproc_t *ip, const char *name));
+int procfs_load_library PROTO((target_t *xp, char *arg));
 
 typedef enum {
 	PROCFS_REG_SP,
