@@ -338,7 +338,7 @@ taddr_t *p_val;
 		}
 #else
 		ur->ur_value = std_ptrace(PTRACE_PEEKUSER, ip->ip_pid,
-							(char *)ur->ur_uaddr, 0);
+							(char *)(int)ur->ur_uaddr, 0);
 #endif
 		if (errno != 0)
 			return -1;
@@ -358,7 +358,7 @@ taddr_t value;
 
 	ur = ip->ip_ptrace_info->uregs + ri;
 	errno = 0;
-	e_ptrace(PTRACE_POKEUSER, ip->ip_pid, (char *)ur->ur_uaddr, (int)value);
+	e_ptrace(PTRACE_POKEUSER, ip->ip_pid, (char *)(int)ur->ur_uaddr, (int)value);
 	ur->ur_is_current = FALSE;
 	return (errno == 0) ? 0 : -1;
 }
