@@ -39,6 +39,8 @@ char ups_ao_ptrace_c_sccsid[] = "@(#)ao_ptrace.c	1.2 04 Jun 1995 (UKC)";
 #include "mreg.h"
 #include "symtab.h"
 #include "target.h"
+#include "st.h"
+#include "ao_syms.h"
 #include "ao_target.h"
 #include "ao_ptrace.h"
 #include "debug.h"
@@ -502,7 +504,10 @@ stopres_t *p_stopres;
 #ifdef AO_ELF
 		xp->xp_new_dynamic_libs = 
 		elf_get_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
-				       &ip->ip_solibs, FALSE);
+				       &ip->ip_solibs,  TRUE, FALSE);
+		/* don't save state */
+		elf_update_dynamic_solibs(xp->xp_apool, xp->xp_textpath,
+					  &ip->ip_solibs, TRUE);
 #endif
 #endif
 	}
@@ -564,24 +569,12 @@ int pid;
 
 	reattach = ptrace_get_last_attach_pid() != 0;
 	if (reattach)
-	{
-	  free_library_load_list();
-	  if (td_init_from())
-	  {
-	    bool eof;
-	    int res;
+	  rescan_init_file();
 
-	    res = td_load_loop(&eof);
-
-	    if (res != 0)
-	      fprintf(stderr, "Error in init file\n");
-	    td_restore_replay_fp();
-	 }
-	}
+	set_current_target(xp);
 #if defined(AO_AOUT_SHLIBS)
 	load_shared_library_symtabs(xp, FALSE);
 #endif
-
 	return 0;
 #else
 	errf("This machine does not support attaching to running processes");
@@ -790,6 +783,15 @@ size_t nbytes;
 }
 
 
+int
+ptrace_load_library(xp, arg)
+target_t *xp;
+char *arg;
+{
+  load_library_input((void *)xp, arg);
+  return -1;
+}
+
 int
 ptrace_write_corefile(ip, name)
 iproc_t *ip;
