@@ -55,6 +55,7 @@ static int get_preload_shlib_list PROTO((alloc_pool_t *ap,
 					 shlib_t **p_shlibs,
 					 int verbose));
 static int get_shlibs_and_global_addrs PROTO((target_t *xp, alloc_pool_t *ap,
+					      alloc_pool_t *shlib_ap,
 				              symtab_t *st,
 					      taddr_t addr_dynamic,
 				              shlib_t **p_shlibs));
@@ -63,7 +64,7 @@ static int load_shlib_symtabs PROTO((shlib_t *shlibs, symtab_t **p_stlist,
 				     target_t *xp));
 static void free_symtab_cache PROTO((target_t *xp));
 static symtab_t *get_symtab_from_cache PROTO((target_t *xp, const char *name));
-static void mark_duplicate_files PROTO((symtab_t *stlist));
+static void mark_duplicate_shlib_files PROTO((symtab_t *stlist));
 static int save_cached_symtab_breakpoints PROTO((target_t *xp));
 static void collapse_srcfile_list PROTO((target_t *xp));
 
@@ -141,9 +142,10 @@ symtab_t **p_next_st;
  *  target has started and the mapping been done.
  */
 static int
-get_shlibs_and_global_addrs(xp, ap, st, addr_dynamic, p_shlibs)
+get_shlibs_and_global_addrs(xp, ap, shlib_ap, st, addr_dynamic, p_shlibs)
 target_t *xp;
 alloc_pool_t *ap;
+alloc_pool_t *shlib_ap;
 symtab_t *st;
 taddr_t addr_dynamic;
 shlib_t **p_shlibs;
@@ -229,7 +231,7 @@ shlib_t **p_shlibs;
 			return -1;
 		if (dgets(xp, SHLIB_NAME(lmbuf), buf, sizeof(buf)) != 0)
 			return -1;
-		if (user_wants_library_loaded(buf))
+		/*		if (user_wants_library_loaded(buf))*/
 		{
 		    if (buf[0] == '.')
 		    {
@@ -249,9 +251,9 @@ shlib_t **p_shlibs;
 			buf[i] = 0;
 			strcat(buf, name);
 		      }
-		    }		  
-		    sh = (shlib_t *)alloc(ap, sizeof(shlib_t));
-		    sh->sh_name = alloc_strdup(ap, buf);
+		    } /* RGA need list latter in get_unloaded_lib */
+		    sh = (shlib_t *)alloc(shlib_ap, sizeof(shlib_t));
+		    sh->sh_name = alloc_strdup(shlib_ap, buf);
 		    sh->sh_addr = SHLIB_ADDR(lmbuf);
 		    sh->sh_next = shlist;
 		    shlist = sh;
@@ -383,8 +385,11 @@ int verbose;
 		    shlist = sh;
 		}
 		  else
+		  {
+		    add_to_noload_list(name, 0);
 		    if (verbose)
 		      errf("\bNOT loading %s", name);
+		  }
 		}
 	      }
 	fclose(fp);
@@ -429,10 +434,10 @@ target_t *xp;
 	if (!GET_IPROC(xp)->ip_symtab_cache)
 	  return 0;
 
-	if ((fp = fopen(get_temp_bpt_filename(), "w")) == NULL)
+	if ((fp = fopen(get_temp_state_filename(), "w")) == NULL)
 	{
 	  errf("Can't create temp saved breakpoints file `%s'",
-	       get_temp_bpt_filename());
+	       get_temp_state_filename());
 	  return 0;
 	}
 	for (st = GET_IPROC(xp)->ip_symtab_cache; st != NULL; st = next) {
@@ -475,7 +480,7 @@ const char *name;
 }
 
 /* RGA more stuff to cope with dynamic relinking of Schlumberger ASAP code */
-static void  mark_duplicate_files(stlist)
+static void  mark_duplicate_shlib_files(stlist)
      symtab_t *stlist;
 {
   symtab_t *st_last, *st, *next;
@@ -576,12 +581,13 @@ int adding_libs_only;
 			fputs("Get shlib names and global addresses...", stderr);
 		ap = alloc_create_pool();
 
-		if (get_shlibs_and_global_addrs(xp, ap, main_st, addr,
-						&shlibs) != 0) {
+		ip->ip_shlibs_ap = alloc_create_pool();
+		if (get_shlibs_and_global_addrs(xp, ap, ip->ip_shlibs_ap, main_st,
+						addr, &shlibs) != 0) {
 			alloc_free_pool(ap);
 			return -1;
 		}
-
+		ip->ip_shlibs = shlibs;
 		if (Verbose == 1)
 			fputs("done\n", stderr);
 		if (load_shlib_symtabs(shlibs, &stlist, xp) != 0) {
@@ -598,7 +604,7 @@ int adding_libs_only;
 		free_symtab_cache(xp);
 		ip->ip_symtab_cache = NULL;
 		ip->ip_shlib_symtabs = stlist;
-		mark_duplicate_files(stlist);
+		mark_duplicate_shlib_files(stlist);
 		if (!adding_libs_only)
 		{
 		  if (saved)
@@ -631,6 +637,8 @@ target_t *xp;
 
 	stlist = NULL;
 	for (sh = shlibs; sh != NULL; sh = sh->sh_next) {
+	  if (!user_wants_library_loaded((char *)sh->sh_name))
+	    continue;
 	  open_ok = 0;
 	  st = NULL;
 	  if (xp != NULL)
@@ -656,9 +664,10 @@ target_t *xp;
 	  {
 	    if (get_message_wn() != -1)
 	      if (st == NULL)
-		errf("\bLoading symtab %s", sh->sh_name);
+		errf("\bLoading symtab %s...", sh->sh_name);
 	      else
-		errf("\bReloading symtab %s", sh->sh_name);
+		errf("\bReloading symtab %s...", sh->sh_name);
+	    add_to_noload_list(sh->sh_name, 1); /* mark as loaded */
 	    if (!open_ok)
 	    {
 	      open_ok = open_for_reading(sh->sh_name, "shared_library",
@@ -667,6 +676,11 @@ target_t *xp;
 		if (fstat(shlib_fd, &stbuf) == 0)
 		  mtime = stbuf.st_mtime;
 	    }
+	    if (get_message_wn() != -1)
+	      if (st == NULL)
+		errf("\bLoading symtab %s...done", sh->sh_name);
+	      else
+		errf("\bReloading symtab %s...done", sh->sh_name);
 	    if (!open_ok)
 	      continue;
 	    if (Verbose == 1)
@@ -714,6 +728,37 @@ target_t *xp;
 	ip->ip_shlib_symtabs = NULL;
 }
 
+void
+free_shlib_list(xp)
+target_t *xp;
+{
+	iproc_t *ip;
+
+	ip = GET_IPROC(xp);
+	
+	alloc_free_pool(ip->ip_shlibs_ap);
+}
+
+char *
+get_unloaded_lib(addr)
+taddr_t addr;
+{
+  iproc_t *ip;
+  shlib_t *sh, *last_sh = NULL;
+  
+  ip = GET_IPROC(get_current_target());
+  for (sh = ip->ip_shlibs; sh != NULL; sh = sh->sh_next)
+  {
+    if (addr > sh->sh_addr)
+      last_sh = sh;
+  }
+  if (last_sh)
+  {
+    return (char *)last_sh->sh_name;
+  }
+  return 0;
+}
+
 #endif /* AO_AOUT_SHLIBS */
 
 #endif /* AO_TARGET */
