@@ -17,3 +17,5 @@ bool aout_next_symtab PROTO((target_t *xp, symtab_t *st, bool load_new,
 			     
 int load_shared_library_symtabs PROTO((target_t *xp, int adding_libs_only));
 void unload_shared_library_symtabs PROTO((target_t *xp));
+char *get_unloaded_lib PROTO((taddr_t addr));
+void free_shlib_list PROTO((target_t *xp));
