@@ -1567,7 +1567,7 @@ ao_get_fu_lnos(f)
   symio_t *symio;
   taddr_t min_addr;
   fil_t *fil, *initial_fil;
-  int symno, symno_start, symno_lim;
+  int symno, symno_start, symno_lim, bogus_last_lno;
   unsigned sline_offset, save_sline_offset;
   unsigned short u;
   Compiler_type compiler;
@@ -1660,19 +1660,26 @@ ao_get_fu_lnos(f)
  
     /*              if (compiler == CT_GNU_CC && fil != f->fu_fil)*/
     /*                      continue;*/
-             
-    lno = (lno_t *)alloc(f->fu_symtab->st_apool,sizeof(lno_t));
-    lno->ln_fil = fil;
+
     /* RGA use unsigned - for files over 32k ! */
     u = (unsigned)nm.n_desc;
+    bogus_last_lno = compiler == CT_CLCC && lno && (max_lnum > (int)u | sline_offset);
+    if (bogus_last_lno)
+      max_lnum = 0;		/* RGA reset */
+    else
+      lno = (lno_t *)alloc(f->fu_symtab->st_apool,sizeof(lno_t));
+    lno->ln_fil = fil;
     lno->ln_num = u | sline_offset;
     lno->ln_addr = addr;
 
     if (lno->ln_num > max_lnum)
       max_lnum = lno->ln_num;
 
-    last->ln_next = lno;
-    last = lno;
+    if (!bogus_last_lno)
+    {
+      last->ln_next = lno;
+      last = lno;
+    }
   }
 
   last->ln_next = NULL;
