@@ -553,6 +553,9 @@ void
 close_symio(si)
 symio_t *si;
 {
+  	if (!si)
+	  return;		/* RGA */
+
 	if (si->fd != -1) {
 		int i;
 		
