@@ -244,11 +244,3 @@ typedef struct hfst {
 	struct hfst *hf_next;
 } hf_t;
 
-/*  Shared library header on Suns.
- */
-typedef struct shlibst {
-	const char *sh_name;
-	taddr_t sh_addr;
-	struct shlibst *sh_next;
-} shlib_t;
-
