@@ -29,10 +29,13 @@ char ups_ao_target_c_sccsid[] = "@(#)ao_target.c	1.1 24/5/95 (UKC)";
 
 #include "ups.h"
 #include "ci.h"
+#include "symtab.h"
 #include "target.h"
 #include "mreg.h"
 #include "dx.h"
 #include "breakpoint.h"
+#include "st.h"
+#include "ao_syms.h"
 #include "ao_core.h"
 #include "ao_target.h"
 #include "ao_text.h"
@@ -78,7 +81,7 @@ static int ao_write_data PROTO((target_t *xp, taddr_t addr, const char *buf,
                                 size_t nbytes));
 static int ao_read_text_from_process PROTO((target_t *xp, taddr_t addr, 
                                             char *buf, size_t nbytes));
-static int ao_write_corefile PROTO((target_t *xp, const char *name));
+static int ao_load_library PROTO((target_t *xp, char *arg));
 static stopres_t ao_restart_child_internal PROTO((target_t *xp, int sig, 
 						  cont_type_t ctype));
 static stopres_t ao_restart_child PROTO((target_t *xp, int sig, 
@@ -104,7 +107,7 @@ static void ao_detach PROTO((target_t *xp));
 #define ps_read_fpreg			procfs_read_fpreg
 #define ps_readreg			procfs_readreg
 #define ps_get_sigstate			procfs_get_sigstate
-#define ps_write_corefile		procfs_write_corefile
+#define ps_load_library			procfs_load_library
 
 #define ps_read_data			procfs_read_data
 #define ps_write_data			procfs_write_data
@@ -128,7 +131,7 @@ static void ao_detach PROTO((target_t *xp));
 #define ps_read_fpreg			ptrace_read_fpreg
 #define ps_readreg			ptrace_readreg
 #define ps_get_sigstate			ptrace_get_sigstate
-#define ps_write_corefile		ptrace_write_corefile
+#define ps_load_library			ptrace_load_library
 
 #define ps_read_data			ptrace_read_data
 #define ps_write_data			ptrace_write_data
@@ -160,7 +163,7 @@ xp_ops_t Ao_ops = {
 	ao_get_retaddr_after_jsr, ao_get_retaddr_after_sig,
 	ao_tswap,
 	ao_read_data, ao_write_data, ao_read_text_from_process,
-	ao_write_corefile, ao_call_func, NULL, ao_do_debug_command
+	ao_load_library, ao_call_func, NULL, ao_do_debug_command
 };
 
 static bool Disable_native = TRUE;
@@ -214,7 +217,7 @@ bool user_gave_core;
 const char **p_cmdline;
 {
 	taddr_t data_addr;
-	int lastsig;
+	int lastsig, ret;
 	iproc_t *ip;
 	Coredesc *co;
 
@@ -238,13 +241,20 @@ const char **p_cmdline;
 	procfs_init(ip);
 #else
 	ip->ip_ptrace_info = NULL;
+#if defined(AO_AOUT_SHLIBS)
+	ip->ip_shlibs_ap = NULL;
+	ip->ip_shlibs = NULL;
+#endif
 #endif
 	
 	ip->ip_solib_addrs = NULL;
 	
-	if (!scan_main_elf_symtab(xp->xp_apool, xp->xp_textpath, textfd,
-				  xp->xp_modtime, &ip->ip_solibs, &ip->ip_solib_addrs,
-				  &xp->xp_mainfunc, &xp->xp_initfunc))
+	ret = scan_main_elf_symtab(xp->xp_apool, xp->xp_textpath, textfd,
+				   xp->xp_modtime, &ip->ip_solibs, &ip->ip_solib_addrs,
+				   &xp->xp_mainfunc, &xp->xp_initfunc,
+				   xp->xp_target_updated);
+	xp->xp_target_updated = FALSE;
+	if (!ret)
 	  return -1;
 
 	if (co != NULL)
@@ -626,11 +636,11 @@ size_t nbytes;
 }
 
 static int
-ao_write_corefile(xp, name)
+ao_load_library(xp, arg)
 target_t *xp;
-const char *name;
+char *arg;
 {
-	return ps_write_corefile(GET_IPROC(xp), name);
+	return ps_load_library(xp, arg);
 }
 
 /*  Breakpoint opcodes for the various machines.
