@@ -17,6 +17,16 @@ typedef struct Procfs_info Procfs_info;
 typedef struct Ptrace_info Ptrace_info;
 #endif
 
+#if defined(AO_AOUT_SHLIBS)
+/*  Shared library header on Suns.
+ */
+typedef struct shlibst {
+	const char *sh_name;
+	taddr_t sh_addr;
+	struct shlibst *sh_next;
+} shlib_t;
+#endif
+
 typedef struct iproc {
 	int ip_pid;
 	struct Coredesc *ip_core;
@@ -38,6 +48,12 @@ typedef struct iproc {
 	symtab_t *ip_shlib_symtabs;
 	symtab_t *ip_symtab_cache;
 #endif
+#if defined(AO_AOUT_SHLIBS)
+	alloc_pool_t *ip_shlibs_ap;
+  	shlib_t *ip_shlibs;
+#endif
 } iproc_t;
 
 #define GET_IPROC(xp)	((iproc_t *)(xp)->xp_data)
+
+
