@@ -7,7 +7,7 @@
  *  unaltered copy of this message (including the copyright).
  */
 
-char ups_as_68020_c_sccsid[] = "@(#)as_68020.c	1.16 26/7/92 (UKC)";
+char ups_as_68020_c_sccsid[] = "@(#)as_68020.c	1.17 16 Apr 1994 (UKC)";
 
 #include <mtrprog/ifdefs.h>
 
@@ -1289,18 +1289,6 @@ int flag;
 	panic("not used with -DTESTING");
 }
 
-static void
-error(fmt, a1, a2, a3, a4, a5)
-const char *fmt;
-int a1, a2, a3, a4, a5;
-{
-	fprintf(stderr, "Fatal internal error: ");
-	fprintf(stderr, fmt, a1, a2, a3, a4, a5);
-	fputs("Aborting... ", stderr);
-	fflush(stderr);
-	abort();
-}
-
 static void
 check_fieldnames_unique()
 {
