@@ -7,27 +7,32 @@
  *  unaltered copy of this message (including the copyright).
  */
 
+/*
+ * parameter of type 'expr_t *' added to code_conversion() and
+ * compile_conversion() so that a proper error message can be generated.
+ * Dibyendu 26/12/98
+ */
 char ups_ci_compile_expr_c_sccsid[] = "@(#)ci_compile_expr.c	1.27 04 Jun 1995 (UKC)";
 
 #define TEST_DATA
 
-
-#include <string.h>
+#include <mtrprog/ifdefs.h>
 
 #include <local/ukcprog.h>
 
 #include "ups.h"
 #include "symtab.h"
+#include "ci.h"
 #include "ci_parse.h"
 #include "ci_types.h"
 #include "ci_constexpr.h"
-#include "ci.h"
 #include "xc_opcodes.h"
 #include "xc_machine.h"
 #include "cx_link.h"
 #include "ci_compile.h"
 #include "ci_compile_expr.h"
 #include "ci_util.h"
+#include "ci_showexpr.h"
 
 static void compile_func_call_expr PROTO((text_t *tx, func_call_expr_t *fce,
 					  typecode_t typecode,
@@ -59,7 +64,8 @@ static addrtype_t unknown_addrtype PROTO((text_t *tx));
 static void code_bitfield_op PROTO((text_t *tx, bitfield_t *bf, opcode_t op));
 static void code_get_whole_bitfield PROTO((text_t *tx, addrtype_t addrtype));
 static addrtype_t compile_conversion PROTO((text_t *tx, unary_expr_t *ue,
-					    expr_context_t context));
+					    expr_context_t context,
+					    expr_t *expr));
 #ifdef TEST_DATA
 static void vtest PROTO((const char *addr, int count));
 #else
@@ -79,7 +85,8 @@ static void initialise_word PROTO((text_t *tx, initialiser_t *in,
 				   loctype_t loctype, char *data,
 				   taddr_t addr));
 static addrtype_t code_conversion PROTO((text_t *tx, addrtype_t addrtype,
-					 optype_t op, typecode_t typecode));
+					 optype_t op, typecode_t typecode,
+					 expr_t *expr));
 static void compile_expr_for_jump PROTO((text_t *tx, expr_t *expr));
 static long short_form PROTO((int func_index));
 
@@ -320,6 +327,13 @@ char *data;
 		vtest(data + addr, sizeof(long));
 		*(long *)(data + addr) = get_initialiser_value(tx, in);
 		break;
+#if WANT_LL
+	case TY_LONGLONG:
+	case TY_ULONGLONG:
+		vtest(data + addr, sizeof(long_long_t));
+		*(long_long_t *)(data + addr) = get_initialiser_value(tx, in);
+		break;
+#endif
 	case TY_FLOAT:
 		if (in->in_inittype != IT_FLOATVAL)
 			ci_panic("it botch in i");
@@ -332,6 +346,14 @@ char *data;
 		vtest(data + addr, sizeof(double));
 		*(double *)(data + addr) = in->in_floatval;
 		break;
+#if WANT_LDBL
+	case TY_LONGDOUBLE:
+		if (in->in_inittype != IT_FLOATVAL)
+			ci_panic("it botch in i");
+		vtest(data + addr, sizeof(long_double_t));
+		*(long_double_t *)(data + addr) = in->in_floatval;
+		break;
+#endif
 	case TY_BITFIELD:
 		{
 			long *p_val, mask, initval;
@@ -464,7 +486,11 @@ type_t *ltype, *rtype;
 		break;
 	}
 
+#if WANT_LDBL
+	if (typecode == TY_FLOAT || typecode == TY_DOUBLE || typecode == TY_LONGDOUBLE) {
+#else
 	if (typecode == TY_FLOAT || typecode == TY_DOUBLE) {
+#endif
 		if (rtype->ty_code != typecode)
 			ci_panic("fp type botch in oto");
 		
@@ -513,6 +539,11 @@ opcode_offset(typecode)
 typecode_t typecode;
 {
 	switch (ci_effective_typecode(typecode)) {
+#if WANT_LL
+	case TY_LONGLONG:
+	case TY_ULONGLONG:
+		return DOUBLE_FORM_OFFSET;	/* FIX: unlikely to be right */
+#endif
 	case TY_LONG:
 	case TY_ULONG:
 	case TY_INT:
@@ -524,6 +555,10 @@ typecode_t typecode;
 		return LONG_FORM_OFFSET;
 	case TY_FLOAT:
 		return FLOAT_FORM_OFFSET;
+#if WANT_LDBL
+	case TY_LONGDOUBLE:
+		return DOUBLE_FORM_OFFSET;	/* FIX: unlikely to be right */
+#endif
 	case TY_DOUBLE:
 		return DOUBLE_FORM_OFFSET;
 	case TY_SHORT:
@@ -847,6 +882,17 @@ expr_context_t context;
 			case TY_DOUBLE:
 				excess_slots += DOUBLE_NSLOTS - 1;
 				break;
+#if WANT_LDBL
+			case TY_LONGDOUBLE:
+				excess_slots += LDOUBLE_NSLOTS - 1;
+				break;
+#endif
+#if WANT_LL
+			case TY_LONGLONG:
+			case TY_ULONGLONG:
+				excess_slots += LONGLONG_NSLOTS - 1;
+				break;
+#endif
 			default:
 				break;
 			}
@@ -880,6 +926,10 @@ expr_context_t context;
 	}
 	ci_code_byte(tx, nargs + excess_slots);
 
+#if WANT_TYPE_PUSHED
+	ci_code_byte(tx, typecode);
+#endif
+
 	bytes_to_pop = tx->tx_sp - save_sp;
 
 	if (bytes_to_pop != 0) {
@@ -893,6 +943,9 @@ expr_context_t context;
 
 	if (context == EC_VALUE) {
 		switch (typecode) {
+		case TY_LONGDOUBLE:
+			ci_code_opcode(tx, OC_PUSH_DOUBLE_RETVAL); /* FIX: */
+			break;
 		case TY_DOUBLE:
 			ci_code_opcode(tx, OC_PUSH_DOUBLE_RETVAL);
 			break;
@@ -1015,6 +1068,11 @@ expr_t *expr;
 	case TY_DOUBLE:
 		ci_code_opcode(tx, OC_CVT_DOUBLE_TO_BOOL);
 		break;
+#if WANT_LDBL
+	case TY_LONGDOUBLE:
+		ci_code_opcode(tx, OC_CVT_LONG_DOUBLE_TO_BOOL);
+		break;
+#endif
 	default:
 		break;
 	}
@@ -1136,7 +1194,7 @@ expr_context_t context;
 		if (left != addrleft) {
 			code_conversion(tx, left_at,
 					left->ex_unary_expr->ue_op,
-					addrleft->ex_type->ty_code);
+					addrleft->ex_type->ty_code, NULL);
 		}
 
 		ci_compile_expression(tx, right, EC_VALUE);
@@ -1146,7 +1204,7 @@ expr_context_t context;
 		if (left != addrleft) {
 			code_conversion(tx, left_at,
 					ci_typecode_to_cvt_op(addrleft->ex_type->ty_code),
-					left->ex_type->ty_code);
+					left->ex_type->ty_code, NULL);
 		}
 
 		ci_code_assign(tx, addrleft->ex_type,
@@ -1314,6 +1372,9 @@ expr_context_t context;
 		addrtype = ci_compile_expression(tx, expr, context);
 		if (context == EC_VALUE) {
 			switch (expr->ex_type->ty_code) {
+			case TY_LONGDOUBLE:
+				opcode = OC_DOUBLE_UNARY_MINUS; /* FIX: */
+				break;
 			case TY_DOUBLE:
 				opcode = OC_DOUBLE_UNARY_MINUS;
 				break;
@@ -1370,10 +1431,11 @@ expr_context_t context;
 }
 
 static addrtype_t
-compile_conversion(tx, ue, context)
+compile_conversion(tx, ue, context, expr)
 text_t *tx;
 unary_expr_t *ue;
 expr_context_t context;
+expr_t *expr;				/* Just for dumping proper error msg */
 {
 	addrtype_t addrtype;
 	typecode_t typecode;
@@ -1385,17 +1447,19 @@ expr_context_t context;
 	
 	typecode = ue->ue_expr->ex_type->ty_code;
 
-	return code_conversion(tx, addrtype, ue->ue_op, typecode);
+	return code_conversion(tx, addrtype, ue->ue_op, typecode, expr);
 }
 
 static addrtype_t
-code_conversion(tx, addrtype, op, typecode)
+code_conversion(tx, addrtype, op, typecode, expr)
 text_t *tx;
 addrtype_t addrtype;
 optype_t op;
 typecode_t typecode;
+expr_t *expr;				/* Just for dumping proper error msg */
 {
 	bool is_integral;
+	const char *what = "";
 
 	is_integral = ci_is_integral(typecode) ||
 					typecode == TY_ENUM ||
@@ -1414,6 +1478,26 @@ typecode_t typecode;
 		return addrtype;
 
 	switch (op) {
+#if WANT_LDBL
+	case OP_CVT_TO_LONG_DOUBLE:
+		if (is_integral) {
+			if (ci_is_signed_type(typecode)) {
+				ci_code_opcode(tx,
+					       OC_CVT_SIGNED_LONG_TO_LONG_DOUBLE);
+			}
+			else {
+				ci_code_opcode(tx,
+					       OC_CVT_UNSIGNED_LONG_TO_LONG_DOUBLE);
+			}
+		}
+		else if (typecode == TY_DOUBLE)
+			ci_code_opcode(tx, OC_CVT_DOUBLE_TO_LONG_DOUBLE);
+		else if (typecode == TY_FLOAT)
+			ci_code_opcode(tx, OC_CVT_FLOAT_TO_LONG_DOUBLE);
+		else if (typecode != TY_LONGDOUBLE)
+			ci_panic("bad type in ctd");
+		break;
+#endif
 	case OP_CVT_TO_DOUBLE:
 		if (is_integral) {
 			if (ci_is_signed_type(typecode)) {
@@ -1425,6 +1509,10 @@ typecode_t typecode;
 					       OC_CVT_UNSIGNED_LONG_TO_DOUBLE);
 			}
 		}
+#if WANT_LDBL
+		else if (typecode == TY_LONGDOUBLE)
+			ci_code_opcode(tx, OC_CVT_LONG_DOUBLE_TO_DOUBLE);
+#endif
 		else if (typecode == TY_FLOAT)
 			ci_code_opcode(tx, OC_CVT_FLOAT_TO_DOUBLE);
 		else if (typecode != TY_DOUBLE)
@@ -1441,20 +1529,31 @@ typecode_t typecode;
 					       OC_CVT_UNSIGNED_LONG_TO_FLOAT);
 			}
 		}
+#if WANT_LDBL
+		else if (typecode == TY_LONGDOUBLE)
+			ci_code_opcode(tx, OC_CVT_LONG_DOUBLE_TO_FLOAT);
+#endif
 		else if (typecode == TY_DOUBLE)
 			ci_code_opcode(tx, OC_CVT_DOUBLE_TO_FLOAT);
 		else if (typecode != TY_FLOAT)
 			ci_panic("bad type in ctf");
 		break;
+#if WANT_LL
+	case OP_CVT_TO_LONG_LONG:
+	case OP_CVT_TO_ULONG_LONG:
+#endif
 	case OP_CVT_TO_PTR:
 	case OP_CVT_TO_LONG:
 	case OP_CVT_TO_ULONG:
 	case OP_CVT_TO_INT:
 	case OP_CVT_TO_UINT:
-	case OP_CVT_TO_SHORT:
-	case OP_CVT_TO_USHORT:
-	case OP_CVT_TO_CHAR:
-	case OP_CVT_TO_UCHAR:
+
+	/* Dibyendu : 'what' set for showing better error message */
+	case OP_CVT_TO_SHORT: 	what = "short"; 		goto doit;
+	case OP_CVT_TO_USHORT: 	what = "unsigned short"; 	goto doit;
+	case OP_CVT_TO_CHAR: 	what = "char"; 			goto doit;
+	case OP_CVT_TO_UCHAR: 	what = "unsigned char";
+doit:
 		/*  If we are converting a non-pointer to a pointer then
 		 *  we don't know the address type of the resulting pointer.
 		 */
@@ -1464,10 +1563,25 @@ typecode_t typecode;
 		/* BUG: we don't cope with the sub-word types.  All we do
 		 *      is make sure the result is integral.
 		 */
-		if (typecode == TY_DOUBLE)
-			ci_code_opcode(tx, OC_CVT_DOUBLE_TO_LONG);
-		else if (typecode == TY_FLOAT)
+		if (typecode == TY_FLOAT)
 			ci_code_opcode(tx, OC_CVT_FLOAT_TO_LONG);
+		else if (typecode == TY_DOUBLE)
+			ci_code_opcode(tx, OC_CVT_DOUBLE_TO_LONG);
+#if WANT_LDBL
+		else if (typecode == TY_LONGDOUBLE)
+			ci_code_opcode(tx, OC_CVT_LONG_DOUBLE_TO_LONG);
+#endif
+		else if (is_integral &&
+			(op == OP_CVT_TO_SHORT || op == OP_CVT_TO_USHORT ||
+			 op == OP_CVT_TO_CHAR  || op == OP_CVT_TO_UCHAR) &&
+			 expr != NULL) {
+		    /* Dibyendu : 26/12/98 : Warning added because this can
+		     * change the meaning of an expression.
+		     */
+			diagf(ET_WARNING, expr->ex_lexinfo,
+				"Conversion to `%s' ignored", what);
+			ci_show_expr_and_type(expr, expr->ex_type);
+		}
 		else if (!is_integral)
 			ci_panic("bad type in cti");
 		break;
@@ -1525,6 +1639,17 @@ expr_context_t context;
 		case TY_DOUBLE:
 			expected_sp = tx->tx_sp + DOUBLE_NSLOTS*sizeof(stackword_t);
 			break;
+#if WANT_LDBL
+		case TY_LONGDOUBLE:
+			expected_sp = tx->tx_sp + LDOUBLE_NSLOTS*sizeof(stackword_t);
+			break;
+#endif
+#if WANT_LL
+		case TY_LONGLONG:
+		case TY_ULONGLONG:
+			expected_sp = tx->tx_sp + LONGLONG_NSLOTS*sizeof(stackword_t);
+			break;
+#endif
 		case TY_STRUCT:
 		case TY_UNION:
 			ci_panic("struct in value context");
@@ -1561,9 +1686,11 @@ expr_context_t context;
 
 	switch(expr->ex_exprtype) {
 	case ET_CAST:
+		addrtype = compile_conversion(tx, expr->ex_unary_expr, context, expr);
+		break;
 	case ET_PROMOTION:
 	case ET_ASSIGNMENT_CONVERSION:
-		addrtype = compile_conversion(tx, expr->ex_unary_expr, context);
+		addrtype = compile_conversion(tx, expr->ex_unary_expr, context, NULL);
 		break;
 	case ET_VAR:
 		addrtype = ci_compile_var_reference(tx,
@@ -1618,6 +1745,16 @@ expr_context_t context;
 				memcpy(buf, (char *)&dval, sizeof(dval));
 				ci_code_opcode(tx, OC_PUSH_DOUBLE_CONST);
 			}
+#if WANT_LDBL
+			else if (expr->ex_type->ty_code == TY_LONGDOUBLE) {
+				long_double_t dval;
+
+				dval = expr->ex_floating_constant_val;
+				count = LDOUBLE_NBYTES;
+				memcpy(buf, (char *)&dval, sizeof(dval));
+				ci_code_opcode(tx, OC_PUSH_DOUBLE_CONST);
+			}
+#endif
 			else {
 				count = 0;	/* to satisfy gcc */
 				ci_panic("non fp type in cce");
