@@ -31,16 +31,18 @@ char ups_ci_constexpr_c_sccsid[] = "@(#)ci_constexpr.c	1.16 16 Apr 1994 (UKC)";
  *
  *	unsigned has the same size and representation as unsigned long
  *
- *	no distinct "long double" type
+ *	no distinct "long double" type  (see note below)
  *
  *	pointers can be cast to int and back without loss of information
  *
  *  It wouldn't take much work to avoid making these assumptions.
  *  You would have to add more members to the constval_t structure,
  *  and add code in various places here to handle the extra cases.
- *  Some of the stuff in n**2 in the number of constval_t members though.
+ *  Some of the stuff is n**2 in the number of constval_t members though.
+ */
+/*  (IDE) I made a start on support for 'long double' but it is not complete
+ *  and does not work.  I suspect my approach may be wrong.
  */
-
 static bool eval_unary_const_expr PROTO((unary_expr_t *ue, const char *what,
 							      constval_t *cv));
 static bool eval_binary_const_expr PROTO((binary_expr_t *be, const char *what,
@@ -117,6 +119,13 @@ constval_t *cv;
 
 	case OP_UNARY_MINUS:
 		switch(ot) {
+#if WANT_LDBL
+			case TY_LONGDOUBLE:	cv->cv_long_double = -val.cv_long_double; break;
+#endif
+#if WANT_LL
+			case TY_LONGLONG:	cv->cv_long_long = -val.cv_long_long; break;
+			case TY_ULONGLONG:	cv->cv_ulong_long = -val.cv_ulong_long; break;
+#endif
 			case TY_DOUBLE:	cv->cv_double = -val.cv_double; break;
 			case TY_FLOAT:	cv->cv_float = -val.cv_float; break;
 			case TY_INT:	cv->cv_int = -val.cv_int; break;
@@ -194,6 +203,16 @@ bool allow_short_types;
 		return TY_FLOAT;
 	case TY_DOUBLE:
 		return TY_DOUBLE;
+#if WANT_LL
+	case TY_LONGLONG:
+		return TY_LONGLONG;
+	case TY_ULONGLONG:
+		return TY_ULONGLONG;
+#endif
+#if WANT_LDBL
+	case TY_LONGDOUBLE:
+		return TY_LONGDOUBLE;
+#endif
 	default:
 		break;
 	}
@@ -203,6 +222,40 @@ bool allow_short_types;
 
 }
 
+#if WANT_LDBL && WANT_LL
+#define RELOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGDOUBLE:	val.cv_int = l.cv_long_double op r.cv_long_double; break; \
+	case TY_LONGLONG:	val.cv_int = l.cv_long_long op r.cv_long_long; break; \
+	case TY_ULONGLONG:	val.cv_int = l.cv_ulong_long op r.cv_ulong_long; break; \
+	case TY_DOUBLE:	val.cv_int = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_int = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_int = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad relop"); break; \
+	}
+#elif WANT_LL
+#define RELOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGLONG:	val.cv_int = l.cv_long_long op r.cv_long_long; break; \
+	case TY_ULONGLONG:	val.cv_int = l.cv_ulong_long op r.cv_ulong_long; break; \
+	case TY_DOUBLE:	val.cv_int = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_int = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_int = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad relop"); break; \
+	}
+#elif WANT_LDBL
+#define RELOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGDOUBLE:	val.cv_int = l.cv_long_double op r.cv_long_double; break; \
+	case TY_DOUBLE:	val.cv_int = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_int = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_int = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad relop"); break; \
+	}
+#else
 #define RELOP(ot,val,l,r,op) \
 	switch(ot) { \
 	case TY_DOUBLE:	val.cv_int = l.cv_double op r.cv_double; break; \
@@ -211,6 +264,7 @@ bool allow_short_types;
 	case TY_UINT:	val.cv_int = l.cv_unsigned op r.cv_unsigned; break; \
 	default:	panic("bad relop"); break; \
 	}
+#endif
 		
 #define INTOP(ot,val,l,r,op) \
 	switch(ot) { \
@@ -219,6 +273,40 @@ bool allow_short_types;
 	default:	panic("bad binop"); break; \
 	}
 
+#if WANT_LL && WANT_LDBL
+#define BINOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGDOUBLE:	val.cv_long_double = l.cv_long_double op r.cv_long_double; break; \
+	case TY_LONGLONG:	val.cv_long_long = l.cv_long_long op r.cv_long_long; break; \
+	case TY_ULONGLONG:	val.cv_ulong_long = l.cv_ulong_long op r.cv_ulong_long; break; \
+	case TY_DOUBLE:	val.cv_double = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_float = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_unsigned = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad binop"); break; \
+	}
+#elif WANT_LL
+#define BINOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGLONG:	val.cv_long_long = l.cv_long_long op r.cv_long_long; break; \
+	case TY_ULONGLONG:	val.cv_ulong_long = l.cv_ulong_long op r.cv_ulong_long; break; \
+	case TY_DOUBLE:	val.cv_double = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_float = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_unsigned = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad binop"); break; \
+	}
+#elif WANT_LDBL
+#define BINOP(ot,val,l,r,op) \
+	switch(ot) { \
+	case TY_LONGDOUBLE:	val.cv_long_double = l.cv_long_double op r.cv_long_double; break; \
+	case TY_DOUBLE:	val.cv_double = l.cv_double op r.cv_double; break; \
+	case TY_FLOAT:	val.cv_float = l.cv_float op r.cv_float; break; \
+	case TY_INT:	val.cv_int = l.cv_int op r.cv_int; break; \
+	case TY_UINT:	val.cv_unsigned = l.cv_unsigned op r.cv_unsigned; break; \
+	default:	panic("bad binop"); break; \
+	}
+#else
 #define BINOP(ot,val,l,r,op) \
 	switch(ot) { \
 	case TY_DOUBLE:	val.cv_double = l.cv_double op r.cv_double; break; \
@@ -227,6 +315,7 @@ bool allow_short_types;
 	case TY_UINT:	val.cv_unsigned = l.cv_unsigned op r.cv_unsigned; break; \
 	default:	panic("bad binop"); break; \
 	}
+#endif
 
 static bool
 eval_binary_const_expr(be, what, cv)
@@ -287,12 +376,25 @@ constval_t *cv;
 	case OP_DIV:
 	case OP_MOD:
 		switch(ot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE:
+			is_zero = r.cv_long_double == 0.0;
+			break;
+#endif
 		case TY_DOUBLE:
 			is_zero = r.cv_double == 0.0;
 			break;
 		case TY_FLOAT:
 			is_zero = r.cv_float == 0.0;
 			break;
+#if WANT_LL
+		case TY_LONGLONG:
+			is_zero = r.cv_long_long == 0.0;
+			break;
+		case TY_ULONGLONG:
+			is_zero = r.cv_ulong_long == 0.0;
+			break;
+#endif
 		case TY_INT:
 			is_zero = r.cv_int == 0;
 			break;
@@ -364,8 +466,23 @@ constval_t *cv;
 	 */
 
 	switch (ci_effective_typecode(oldot)) {
+#if WANT_LDBL
+	case TY_LONGDOUBLE:
+		switch (newot) {
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_long_double; break;
+		case TY_DOUBLE: newval.cv_double = cv->cv_long_double; break;
+		case TY_FLOAT: newval.cv_float = cv->cv_long_double; break;
+		case TY_INT: newval.cv_int = cv->cv_long_double; break;
+		case TY_UINT: newval.cv_unsigned = cv->cv_long_double; break;
+		default: panic("duc botch"); break;
+		}
+		break;
+#endif
 	case TY_DOUBLE:
 		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_double; break;
+#endif
 		case TY_DOUBLE: newval.cv_double = cv->cv_double; break;
 		case TY_FLOAT: newval.cv_float = cv->cv_double; break;
 		case TY_INT: newval.cv_int = cv->cv_double; break;
@@ -382,6 +499,9 @@ constval_t *cv;
 		break;
 	case TY_FLOAT:
 		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_float; break;
+#endif
 		case TY_DOUBLE: newval.cv_double = cv->cv_float; break;
 		case TY_FLOAT: newval.cv_float = cv->cv_float; break;
 		case TY_INT: newval.cv_int = cv->cv_float; break;
@@ -398,8 +518,15 @@ constval_t *cv;
 	case TY_BOOLEAN:
 	case TY_INT:
 		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_int; break;
+#endif
 		case TY_DOUBLE: newval.cv_double = cv->cv_int; break;
 		case TY_FLOAT: newval.cv_float = cv->cv_int; break;
+#if WANT_LL
+		case TY_LONGLONG: newval.cv_long_long = cv->cv_int; break;
+		case TY_ULONGLONG: newval.cv_ulong_long = cv->cv_int; break;
+#endif
 		case TY_INT: newval.cv_int = cv->cv_int; break;
 		case TY_UINT: newval.cv_unsigned = cv->cv_int; break;
 		default: panic("duc botch"); break;
@@ -407,13 +534,50 @@ constval_t *cv;
 		break;
 	case TY_UINT:
 		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_unsigned; break;
+#endif
 		case TY_DOUBLE: newval.cv_double = cv->cv_unsigned; break;
 		case TY_FLOAT: newval.cv_float = cv->cv_unsigned; break;
+#if WANT_LL
+		case TY_LONGLONG: newval.cv_long_long = cv->cv_unsigned; break;
+		case TY_ULONGLONG: newval.cv_ulong_long = cv->cv_unsigned; break;
+#endif
 		case TY_INT: newval.cv_int = cv->cv_unsigned; break;
 		case TY_UINT: newval.cv_unsigned = cv->cv_unsigned; break;
 		default: panic("duc botch"); break;
 		}
 		break;
+#if WANT_LL
+	case TY_LONGLONG:
+		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_long_long; break;
+#endif
+		case TY_DOUBLE: newval.cv_double = cv->cv_long_long; break;
+		case TY_FLOAT: newval.cv_float = cv->cv_long_long; break;
+		case TY_LONGLONG: newval.cv_long_long = cv->cv_long_long; break;
+		case TY_ULONGLONG: newval.cv_ulong_long = cv->cv_long_long; break;
+		case TY_INT: newval.cv_int = cv->cv_long_long; break;
+		case TY_UINT: newval.cv_unsigned = cv->cv_long_long; break;
+		default: panic("duc botch"); break;
+		}
+		break;
+	case TY_ULONGLONG:
+		switch (newot) {
+#if WANT_LDBL
+		case TY_LONGDOUBLE: newval.cv_long_double = cv->cv_ulong_long; break;
+#endif
+		case TY_DOUBLE: newval.cv_double = cv->cv_ulong_long; break;
+		case TY_FLOAT: newval.cv_float = cv->cv_ulong_long; break;
+		case TY_LONGLONG: newval.cv_long_long = cv->cv_ulong_long; break;
+		case TY_ULONGLONG: newval.cv_ulong_long = cv->cv_ulong_long; break;
+		case TY_INT: newval.cv_int = cv->cv_ulong_long; break;
+		case TY_UINT: newval.cv_unsigned = cv->cv_ulong_long; break;
+		default: panic("duc botch"); break;
+		}
+		break;
+#endif
 	default:
 		panic("oldot botch in duc"); break;
 	}
