@@ -9,10 +9,7 @@
 
 char ups_ci_opcodes_c_sccsid[] = "@(#)ci_opcodes.c	1.22 16 Sep 1994 (UKC)";
 
-#include <sys/types.h>
-#include <stdio.h>
-#include <ctype.h>
-#include <string.h>
+#include <mtrprog/ifdefs.h>
 
 #include <local/ukcprog.h>
 #include <mtrprog/utils.h>
@@ -20,9 +17,9 @@ char ups_ci_opcodes_c_sccsid[] = "@(#)ci_opcodes.c	1.22 16 Sep 1994 (UKC)";
 
 #include "ups.h"
 #include "symtab.h"
+#include "ci.h"
 #include "ci_parse.h"
 #include "ci_util.h"
-#include "ci.h"
 #include "xc_opcodes.h"
 #include "xc_machine.h"
 #include "cx_link.h"
@@ -48,7 +45,7 @@ static void dump_linknames PROTO((FILE *fp, const char *what,
 					linkname_t *list, int *p_count));
 static linkname_t *index_to_linkname PROTO((linkinfo_t *li, int var_index));
 static void show_call PROTO((FILE *fp, linkinfo_t *li, int func_index,
-						int nargs, int optype));
+					int nargs, int typecode, int optype));
 static void dump_ext_relocs PROTO((FILE *fp, const char *what,
 							ext_reloc_t *er));
 static void dump_data PROTO((FILE *fp, const char *what,
@@ -665,14 +662,17 @@ int switchval;
 			ci_panic("bad opcode");
 			val = 0;	/* to satisfy gcc */
 		}
-		if (val == switchval)
+		if (val == switchval) {
+			/* Dibyendu : this was missing */
+			pc += (short)GETWORD(pc);
 			break;
+		}
 		pc += 2;
 	}
 	if (i == ncase)
 		pc += (short)GETWORD(pc);
 
-	return 0;
+	return pc;
 }
 
 int
@@ -706,7 +706,7 @@ textword_t **p_pc;
 	case OC_SWITCH_ON_CHAIN_B:
 	case OC_SWITCH_ON_CHAIN_W:
 	case OC_SWITCH_ON_CHAIN_L:
-		pc = get_pc_after_switch_on_chain(pc, (int)ma->ma_sp[0]);
+		pc = get_pc_after_switch_on_chain(pc, (int)(ma->ma_sp[0]));
 		break;
 
 	case OC_PUSH_FLOAT_CONST:
@@ -790,7 +790,9 @@ textword_t **p_pc;
 
 	pc = *p_pc;
 
-	if (func_index >= 0) {
+	/* Dibyendu : following looks incorrect - see xc_machine.c */
+	/* if (func_index >= 0) */
+	{
 		extref_t *er;
 
 		if (func_index > 0) {
@@ -799,7 +801,9 @@ textword_t **p_pc;
 			return 0;
 		}
 
-		er = fcf->cf_funcrefs[-func_index];
+		/* Dibyendu : following looks incorrect - see xc_machine.c */
+		/* er = fcf->cf_funcrefs[-func_index]; */
+		er = fcf->cf_funcrefs[func_index];
 
 		if (er->er_file != NULL) {
 			*p_cf = er->er_file;
@@ -812,7 +816,7 @@ textword_t **p_pc;
 		}
 	}
 
-	*p_pc += operand_size + 1;
+	*p_pc += operand_size + 1 + WANT_TYPE_PUSHED;
 	return 0;
 }
 
@@ -836,17 +840,31 @@ textword_t *text;
 	case OS_OTHER:
 		switch (opcode) {
 		case OC_CALL_B:
-			return sizeof(char) + 1;
+			return sizeof(char) + 1 + WANT_TYPE_PUSHED;
 		case OC_CALL_W:
-			return sizeof(short) + 1;
+			return sizeof(short) + 1 + WANT_TYPE_PUSHED;
 		case OC_CALL_L:
-			return sizeof(long) + 1;
+			return sizeof(long) + 1 + WANT_TYPE_PUSHED;
+#if WANT_TYPE_PUSHED
+		case OC_CALL_INDIRECT:
+			return sizeof(char) + WANT_TYPE_PUSHED;
+#endif
+#if WANT_OLD_LINK
 		case OC_LINK_B:
 			return sizeof(char) + 1;
 		case OC_LINK_W:
 			return sizeof(short) + 1;
 		case OC_LINK_L:
 			return sizeof(long) + 1;
+#else
+		/* Fix LINK ARG (CHAR -> LONG) : Dibyendu */
+		case OC_LINK_B:
+			return sizeof(char) + 4;
+		case OC_LINK_W:
+			return sizeof(short) + 4;
+		case OC_LINK_L:
+			return sizeof(long) + 4;
+#endif
 		case OC_PUSH_FLOAT_CONST:
 			return FLOAT_NBYTES;
 		case OC_PUSH_DOUBLE_CONST:
@@ -890,6 +908,7 @@ textword_t *text, *pc;
 opcode_t opcode;
 {
 	int minval, ncase, i;
+	int ival;
 	float fval;
 	double dval;
 
@@ -952,16 +971,35 @@ opcode_t opcode;
 		fprintf(fp, "%f", dval);
 		break;
 
+	/* Fix WANT_TYPE_PUSHED : Dibyendu */
+#if WANT_TYPE_PUSHED
 	case OC_CALL_B:
-		show_call(fp, li, *pc, pc[1], 'b');
+		show_call(fp, li, *pc, pc[1], pc[2], 'b');
 		break;
 	case OC_CALL_W:
-		show_call(fp, li, (int)GETWORD(pc), pc[2], 'w');
+		show_call(fp, li, (int)GETWORD(pc), pc[2], pc[3], 'w');
 		break;
 	case OC_CALL_L:
-		show_call(fp, li, (int)GETLONG(pc), pc[4], 'l');
+		show_call(fp, li, (int)GETLONG(pc), pc[4], pc[5], 'l');
 		break;
+#else
+	case OC_CALL_B:
+		show_call(fp, li, *pc, pc[1], 0, 'b');
+		break;
+	case OC_CALL_W:
+		show_call(fp, li, (int)GETWORD(pc), pc[2], 0, 'w');
+		break;
+	case OC_CALL_L:
+		show_call(fp, li, (int)GETLONG(pc), pc[4], 0, 'l');
+		break;
+#endif
+#if WANT_TYPE_PUSHED
+	case OC_CALL_INDIRECT:
+		fprintf(fp, "#%d %s", *pc, ci_typecode_to_name(pc[1]));
+		break;
+#endif
 
+#if WANT_OLD_LINK
 	case OC_LINK_B:
 		fprintf(fp, "%d.b,%d", *pc, pc[1]);
 		break;
@@ -971,6 +1009,22 @@ opcode_t opcode;
 	case OC_LINK_L:
 		fprintf(fp, "%d.l,%d", GETLONG(pc), pc[4]);
 		break;
+#else
+	case OC_LINK_B:
+		ival = *pc++;
+		fprintf(fp, "%d.b,%d", ival, GETLONG(pc));
+		break;
+	case OC_LINK_W:
+		ival = GETWORD(pc);
+		pc += 2;
+		fprintf(fp, "%d.w,%d", ival, GETLONG(pc));
+		break;
+	case OC_LINK_L:
+		ival = GETLONG(pc);
+		pc += 4;
+		fprintf(fp, "%d.l,%d", ival, GETLONG(pc));
+		break;
+#endif
 
 	case OC_UNRESOLVED_JUMP:
 	case OC_JUMP:
@@ -993,27 +1047,32 @@ opcode_t opcode;
 }
 
 static void
-show_call(fp, li, func_index, nargs, optype)
+show_call(fp, li, func_index, nargs, typecode, optype)
 FILE *fp;
 linkinfo_t *li;
-int func_index, nargs, optype;
+int func_index, nargs, typecode, optype;
 {
 	libfunc_t *lf;
+#if WANT_TYPE_PUSHED
+	char *tn = (char *)ci_typecode_to_name((typecode_t)typecode);
+#else
+	char *tn = "";
+#endif
 
 	if (li == NULL || (li->li_nsymfuncs == 0 && func_index > 0)) {
-		fprintf(fp, "%d.%c #%d", func_index, optype, nargs);
+		fprintf(fp, "%d.%c #%d %s", func_index, optype, nargs, tn);
 		return;
 	}
 
 	if (func_index > 0) {
-		fprintf(fp, "%s #%d",
-			    li->li_symfuncs[func_index - 1].sf_name, nargs);
+		fprintf(fp, "%s #%d %s",
+			li->li_symfuncs[func_index - 1].sf_name, nargs, tn);
     		return;
 	}
 
 	for (lf = li->li_undef_funcs; lf != NULL; lf = lf->lf_next) {
 		if (lf->lf_index == func_index) {
-			fprintf(fp, "%s #%d", lf->lf_name, nargs);
+			fprintf(fp, "%s #%d %s", lf->lf_name, nargs, tn);
 			return;
 		}
 	}
