@@ -96,7 +96,7 @@ ci_panic(s)
 const char *s;
 {
 	if (ci_Catching_panics) {
-		errf("Compiler error: %s (please tell mtr@ukc.ac.uk)", s);
+		errf("Compiler error: %s", s);
 		longjmp(ci_Catch_panic_env, 1);
 	}
 	else {
