@@ -21,7 +21,8 @@ void load_config PROTO((const char *state_path, const char *user_path,
 
 int read_config_file PROTO((const char *path, bool from_statefile,
 			    bool must_exist, bool breakpoints_only,
-			    bool ignore_breakpoints, bool *p_want_auto_start));
+			    bool ignore_breakpoints, bool *p_want_auto_start,
+			    bool no_statefile_errors));
 
 void load_state_file PROTO((void));
 
