@@ -258,12 +258,13 @@ int lnum;
 }
 
 int
-show_var(sw, fil, lnum, wholename, out_f)
+show_var(sw, fil, lnum, wholename, out_f, control_down)
 Srcwin *sw;
 fil_t *fil;
 int lnum;
 const char *wholename;
 errf_ofunc_t out_f;
+int control_down;
 {
 	func_t *f;
 	char **names, **nptr;
@@ -317,7 +318,8 @@ errf_ofunc_t out_f;
 		oldf = errf_set_ofunc(out_f);
 	      if (!find_func_by_name(class_name, &f0, &f1))
 	      {
-		show_func_source(sw, f0);
+		if (f0)
+		  show_func_source(sw, f0);
 		have_var = 1;
 	      }
 	      else
@@ -331,8 +333,15 @@ errf_ofunc_t out_f;
 					 (common_block_t *)NULL, 0, &f0, &f1,
 					 &v, &v1, &cblock, &fil0, &fil1))
 		{
-		  show_func_source(sw, f0);
-		  have_var = 1;
+		  if (f0)
+		    show_func_source(sw, f0);
+		  else
+		    if (v)
+		      have_var = show_global
+			(sw, fil, f, (common_block_t *)NULL,
+			 v->va_name, TRUE, FALSE, FALSE, (objid_t *)NULL, FALSE);
+		  else
+		    have_var = 0;
 		}
 	      }
 	      free(class_name);
@@ -344,7 +353,7 @@ errf_ofunc_t out_f;
 
 	if (have_var != 1)
 	{
-	  if (can_get_target_vars(get_current_target())) {
+	  if (!control_down && can_get_target_vars(get_current_target())) {
 	    if (f == NULL ||
 		!(have_var = show_local(sw, f, lnum, names[0])))
 	    {
@@ -388,7 +397,8 @@ errf_ofunc_t out_f;
 	    oldf = errf_set_ofunc(out_f);
 	  if (method && !find_func_by_name(class_name, &f0, &f1))
 	  {
-	    show_func_source(sw, f0);
+	    if (f0)
+	      show_func_source(sw, f0);
 	    have_var = 1;
 	  }
 	  else
@@ -1199,14 +1209,14 @@ objid_t *p_var_obj;
 		obj = (objid_t)cblock;
 	}
 	else if (fil != NULL) {
-		add_source_file_object_if_required(fil, FALSE, FALSE);
+		add_source_file_object_if_required(fil, FALSE, FALSE, FALSE);
 		obj = (objid_t)fil;
 	}
 	else
 		obj = GLOBALS_OBJCODE;
 	
        if (fil1 != NULL) {
-		add_source_file_object_if_required(fil1, FALSE, FALSE);
+		add_source_file_object_if_required(fil1, FALSE, FALSE, FALSE);
 		obj1 = (objid_t)fil1;
 	}
 
@@ -1261,7 +1271,7 @@ var_t *v;
   objid_t obj;
 
   if (fil != NULL) {
-    add_source_file_object_if_required(fil, FALSE, FALSE);
+    add_source_file_object_if_required(fil, FALSE, FALSE, FALSE);
     obj = (objid_t)fil;
   }
   else
