@@ -24,7 +24,7 @@ bool show_local_expr PROTO((fil_t *fil, int lnum, const char *text));
 #ifdef SRCWIN_H_INCLUDED
 void show_var_from_typing_line PROTO((Srcwin *sw, const char *name));
 int show_var PROTO((Srcwin *sw, fil_t *fil, int lnum, const char *wholename,
-		     errf_ofunc_t out_f));
+		     errf_ofunc_t out_f, int control_down));
 
 #ifdef ST_H_INCLUDED
 bool show_global PROTO((Srcwin *sw, fil_t *srcfil, func_t *srcfunc,
