@@ -41,6 +41,7 @@
 #define MR_VAR_SHOW_STATICS		'S' /* RCB */
 #define MR_VAR_ACCELERATOR_ACTION	'q' /* RCB */
 #define MR_VAR_DUMP_TO_OUTWIN		'W' /* RCB */
+#define MR_VAR_SHOW_SIZE		'X' /* RGA */
 
 /*  Return values from menus/srcmen.c
  */
@@ -122,6 +123,10 @@
 #define MR_HIDE_SOURCE_FILES_COMPLETELY 'C'
 #define MR_ADD_SOURCE_PATH		'a'
 
+/*  Return values from menus/nolibmen.c
+ */
+#define MR_LOAD_LIBRARY			'l'
+
 /*  Return values from menus/varmen.c
  */
 #define MR_EXPAND_LIKE_BEFORE		'B'
@@ -234,7 +239,7 @@
 
 /*  Return values from menus/coremen.c
  */
-#define MR_WRITE_CORE			'c'
+#define MR_LOAD_ONE_LIBRARY		'Z'
 #define MR_SAVE_STATE			's'
 #define MR_LOAD_STATE			'l'
 #define MR_RESCAN_INIT_FILE		'i'
@@ -251,6 +256,8 @@
 #define MR_TGT_DETACH			'w'
 #define MR_TGT_KILL			'k'
 #define MR_TGT_DONT_KILL		'?'
+#define MR_TGT_INIT			'i' /* RGA used when editing target name
+					     - not a menu option */
 
 /*  Return values from menus/envheadmen.c
  */
