@@ -16,6 +16,7 @@ char ups_obj_bpt_c_sccsid[] = "@(#)obj_bpt.c	1.42 24 May 1995 (UKC)";
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 
 #include <local/wn.h>
@@ -48,6 +49,7 @@ char ups_obj_bpt_c_sccsid[] = "@(#)obj_bpt.c	1.42 24 May 1995 (UKC)";
 #include "expr.h"
 #include "exec.h"
 #include "obj_buildf.h"
+#include "va.h"
 
 typedef struct bpdescst {
 	Editblock *editblock;
@@ -192,7 +194,7 @@ bool from_statefile;
 		lastpath = strsave(path);
 	}
 
-	read_config_file(path, from_statefile, TRUE, TRUE, FALSE, &junk);
+	read_config_file(path, from_statefile, TRUE, TRUE, FALSE, &junk, FALSE);
 }
 
 static bool
@@ -390,9 +392,13 @@ int *p_lnum;
 		remove_object(obj, OBJ_SELF);
 	
 	if ((obj = add_breakpoint_object(f, fil, lnum)) == NULL) {
-		if (have_code)
-			skip_to_close_curly(fp, p_lnum);
-		return -1;
+	  /* RGA try again without a file. Bpts like purify_stop_here
+	     can have bogus files */
+	  if ((obj = add_breakpoint_object(f, 0, -1)) == NULL) {
+	    if (have_code)
+	      skip_to_close_curly(fp, p_lnum);
+	    return -1;
+	  }
 	}
 	if (activationstr == NULL)
 	  set_bpt_state(obj, FALSE, FALSE); /* default is active */
@@ -1896,12 +1902,101 @@ save_matching_bpts(st, fil, handle)
 	}
 }
 
+static void
+null_ofunc(s)
+const char *s;
+{
+}
+
+int
+save_target_state(xp, remove_bpts, remove_vars, append)
+target_t *xp;
+bool remove_bpts;
+bool remove_vars;
+bool append;
+{
+  FILE *fp;
+  int res = 0;
+  char *state_path;
+
+  state_path = get_temp_state_filename();
+
+  if ((fp = fopen(state_path, append ? "a" : "w")) == NULL)
+  {
+    errf("Can't create temp state file `%s'",
+	 state_path);
+    return 0;
+  }
+  if (target_process_exists(xp))
+    save_var_display_state(TRUE, TRUE);
+
+  if (remove_bpts == TRUE)
+    res = save_all_breakpoints_to_file(state_path, fp); 
+		
+  if (res == 0)
+    res = write_var_state_to_file(state_path, fp);
+		
+  if (ferror(fp) || fflush(fp) == EOF) 
+    errf("Error writing to `%s' ups state file", state_path);
+
+  fclose(fp);
+  if (remove_bpts == TRUE)
+    remove_all_breakpoints();
+  if (remove_vars == TRUE)
+  {
+    remove_all_vars();
+    free_vslist();
+    free_vflist();
+    free_ftlist();
+  }
+  return 1;
+}
+
 void
-restore_cached_bpts()
+restore_target_state(remove_vars, remove_bpts)
+bool remove_vars;
+bool remove_bpts;
 {
   bool junk;
+  struct stat stat_buf;
+  char *state_path;
+  int res = 0;
+  FILE *fp;
+  errf_ofunc_t oldf, oldf1;
 
-  read_config_file(get_temp_bpt_filename(), TRUE, TRUE, TRUE, FALSE, &junk);
+  state_path = get_temp_state_filename();
+  if ( stat(state_path, &stat_buf) != 0)
+    return;
+
+  if (remove_vars == TRUE)
+  {
+    remove_all_vars();
+    free_vslist();
+    free_vflist();
+    free_ftlist();
+  }
+  if (Breakpoint_header_exists == TRUE)
+  {
+    if ((fp = fopen(state_path, "a")) == NULL)
+    {
+      errf("Can't append to temp state file `%s'",
+	   state_path);
+      return;
+    }
+    save_all_breakpoints_to_file(state_path, fp); 
+    if (ferror(fp) || fflush(fp) == EOF) 
+      errf("Error writing to `%s' ups state file", state_path);
+
+    fclose(fp);
+    if (remove_bpts == TRUE)
+      remove_all_breakpoints();
+  }
+  /* suprres messages because the target may be completely different */
+  oldf = errf_set_ofunc(null_ofunc);
+  oldf1 = errf_set_ofunc1(null_ofunc);
+  read_config_file(get_temp_state_filename(), TRUE,  FALSE, FALSE, FALSE, &junk, TRUE);
+  errf_set_ofunc(oldf);
+  errf_set_ofunc1(oldf1);
 }
 
 /*  Add the breakpoint header to the display.  Called on startup from main().
@@ -1945,7 +2040,7 @@ char *arg;
 			errf("No saved state file to load breakpoints from");
 		else
 			read_config_file(Statefile_path, TRUE, TRUE, TRUE,
-					 FALSE, &junk);
+					 FALSE, &junk, FALSE);
 		break;
 	case MR_LOAD_BREAKPOINTS_EXC:
 		load_breakpoints(FALSE);
@@ -2034,12 +2129,6 @@ long bp_data;
     return 0;
 }
 
-static void
-null_ofunc(s)
-const char *s;
-{
-}
-
 void
 remove_all_breakpoints()
 {
@@ -2055,22 +2144,6 @@ remove_all_breakpoints()
   }
 }
 
-void
-save_all_breakpoints( handle)
-     char *handle;
-{
-  objid_t obj, next;
-  FILE *fp;
-  
-  fp = (FILE *)handle;
-  if (Breakpoint_header_exists == FALSE)
-    return;		/* RGA */
-  for (obj = get_code(BPHEAD_OBJCODE, OBJ_CHILD); obj != NULL; obj = next) {
-    next = get_code(obj, OBJ_NEXT);
-    save_object_to_file(NULL, fp, (bpdesc_t *)obj);
-  }
-}
-
 int
 add_breakpoints_for_fname( fname, pfirst)
 const char* fname;
@@ -2147,3 +2220,9 @@ long *p_fg, *p_bg;
     *p_fg = color.co_pixel;
   *p_bg = WN_BG(wn);
 }
+ 
+void
+src_window_redraw()
+{
+  srcwin_redraw(get_current_srcwin());
+}
