@@ -11,12 +11,13 @@
 extern struct Edit_history* bpt_history;
 
 #ifdef TARGET_H_INCLUDED
+int save_target_state PROTO((target_t *xp, bool remove_bpts,
+			     bool remove_vars, bool append));
 void recalculate_bpt_addrs PROTO((target_t *xp));
 bool execute_bp_code PROTO((breakpoint_t *bp, taddr_t fp, taddr_t ap));
 #endif
 
 int save_all_breakpoints_to_file PROTO((const char *path, FILE *fp));
-void save_all_breakpoints PROTO((char *handle));
 
 int breakpoint_is_disabled PROTO((long bp_data));
 
@@ -82,3 +83,6 @@ extern fdef_t Bpt_fdefs[];
 extern fnamemap_t Bpt_fnamemap[];
 #endif
 
+void restore_target_state PROTO((bool remove_vars, bool remove_bpts));
+void reset_filesize_operations PROTO((void));
+void src_window_redraw PROTO((void));
