@@ -222,7 +222,7 @@ get_environment()
 	int nenv, i;
 
 	if (Envtab == NULL)
-		return environ;
+		return (const char **)environ;
 	
 	nenv = 1;	/* For the final NULL */
 	for (ee = Envtab->ee_next; ee != Envtab; ee = ee->ee_next)
@@ -257,7 +257,7 @@ make_envtab()
 	Envtab = (enventry_t *)e_malloc(sizeof(enventry_t));
 	Envtab->ee_next = Envtab->ee_prev = Envtab;
 
-	for (envp = environ; *envp != NULL; ++envp) {
+	for (envp = (const char **)environ; *envp != NULL; ++envp) {
 		enventry_t *ee;
 
 		ee = (enventry_t *)e_malloc(sizeof(enventry_t));
