@@ -59,8 +59,11 @@ static void expand_module PROTO((objid_t par));
 static void add_module_object_if_necessary PROTO((Module *module));
 static int modfunc_cmp PROTO((objid_t obj1, objid_t obj2));
 static void set_max_module_width PROTO((Module *module));
+static void fil_note_file_collapsed PROTO((fil_t *fil, bool flag));
 static void update_displayed_file_entries PROTO((objid_t par));
+static int base_src_cmp PROTO((objid_t obj1, objid_t obj2, bool no_hide));
 static int src_cmp2 PROTO((objid_t obj1, objid_t obj2));
+static int src_cmp3 PROTO((objid_t obj1, objid_t obj2));
 
 const char Srchead_format[] = "Source files\n";
 
@@ -94,10 +97,6 @@ const char Module_func_format[] = "\n%[-]10cs %[-]s\n";
 static int Max_src_width = 0;
 static int Max_module_width = 0;
 
-/*  Horizontal gap in pixels between objects.
- */
-#define OBJECT_HGAP	10	/* RGA not used now */
-
 char *
 header_format_obj(code)
 objid_t code;
@@ -422,8 +421,9 @@ objid_t obj1, obj2;
 }
 
 static int
-src_cmp2(obj1, obj2)
+base_src_cmp(obj1, obj2, no_hide)
      objid_t obj1, obj2;
+     bool no_hide;
 {
   int strs_differ;
   
@@ -441,13 +441,27 @@ src_cmp2(obj1, obj2)
   if (!strs_differ && (((fil_t *)obj1)->fi_flags & FI_DUPLICATE ||
 		       ((fil_t *)obj2)->fi_flags & FI_DUPLICATE))
     return 1;
- if (!strs_differ && 
+ if (no_hide == FALSE && !strs_differ && 
       (((fil_t *)obj1)->fi_flags & FI_HIDE_ENTRY) == 0 &&
       (((fil_t *)obj2)->fi_flags & FI_HIDE_ENTRY) == 0)
     ((fil_t *)obj2)->fi_flags |= FI_HIDE_ENTRY;
   return strs_differ;
 }
 
+static int
+src_cmp2(obj1, obj2)
+     objid_t obj1, obj2;
+{
+  return base_src_cmp(obj1, obj2, FALSE);
+}
+
+
+static int
+src_cmp3(obj1, obj2)
+     objid_t obj1, obj2;
+{
+  return base_src_cmp(obj1, obj2, TRUE);
+}
 
 static int
 module_cmp(obj1, obj2)
@@ -489,7 +503,7 @@ char *arg;
 		break;
 	case MR_HIDE_SOURCE_FILES_COMPLETELY:
 		update_displayed_file_entries(obj);
-		iterate_over_source_files(note_file_collapsed, FALSE);
+		iterate_over_source_files(fil_note_file_collapsed, FALSE);
 		remove_object(obj, OBJ_DESCENDENTS);
 		break;
 	case MR_ADD_SOURCE_PATH:
@@ -503,6 +517,14 @@ char *arg;
 	}
 }
 
+static void
+fil_note_file_collapsed(fil, flag)
+fil_t *fil;
+bool flag;
+{
+  note_file_collapsed((objid_t)fil, flag);
+}
+  
 static void
 update_displayed_file_entries(par)
 objid_t par;
@@ -517,19 +539,30 @@ objid_t par;
 }
 	
 void
-add_source_file_object_if_required(fil, restore_file_displays, use_src_cmp)
+add_source_file_object_if_required(fil, restore_file_displays,
+				   use_src_cmp2, use_src_cmp3)
 fil_t *fil;
 bool restore_file_displays;
-bool use_src_cmp;
+bool use_src_cmp2;
+bool use_src_cmp3;
 {
-  if (use_src_cmp)
+  if (fil->fi_symtab && fil->fi_symtab->st_eclipsed)
+    return;
+
+  if (use_src_cmp2)
   {
     if (obj_child_exists2(SRCHEAD_OBJCODE, (objid_t)fil, src_cmp2))
       return;
   }
   else
-    if (obj_child_exists(SRCHEAD_OBJCODE, (objid_t)fil))
-      return;
+    if (use_src_cmp3)
+    {
+      if (obj_child_exists2(SRCHEAD_OBJCODE, (objid_t)fil, src_cmp3))
+	return;
+    }
+    else
+      if (obj_child_exists(SRCHEAD_OBJCODE, (objid_t)fil))
+	return;
   
   new_object((objid_t)fil, OT_SFILE, SRCHEAD_OBJCODE, OBJ_CHILD);
   set_field_value((objid_t)fil, FN_SFILE_NAME, (fval_t)fil->fi_name_only);
@@ -543,7 +576,15 @@ add_source_file_object_if_necessary(fil, restore_file_displays)
      fil_t *fil;
      bool restore_file_displays;
 {
-  add_source_file_object_if_required(fil, restore_file_displays, TRUE);
+  add_source_file_object_if_required(fil, restore_file_displays, TRUE, FALSE);
+}
+
+void
+add_source_file_object_if_necessary2(fil, restore_file_displays)
+     fil_t *fil;
+     bool restore_file_displays;
+{
+  add_source_file_object_if_required(fil, restore_file_displays, FALSE, TRUE);
 }
 
 static void
@@ -645,6 +686,28 @@ objid_t par;
 		sort_children(par, modfunc_cmp);
 }
 
+static 
+void
+srcfile_getsize_internal(unused_obj, unused_par, sz, reset)
+objid_t unused_obj, unused_par;
+struct szst *sz;
+bool reset;
+{
+	static bool first_call = TRUE;
+
+	if (reset)
+	{
+	  first_call = TRUE;
+	  return;
+	}
+	if (first_call) {
+		iterate_over_source_files(set_max_src_width, FALSE);
+		first_call = FALSE;
+	}
+	sz->sz_depth = wn_get_sysfont()->ft_height;
+	sz->sz_width = Max_src_width + wn_get_sysfont()->ft_width;
+}
+
 /*  Return the width in pixels of a source file object.
  *
  *  A pointer to this function is passed to the obj library.
@@ -655,40 +718,38 @@ srcfile_getsize(unused_obj, unused_par, sz)
 objid_t unused_obj, unused_par;
 struct szst *sz;
 {
-	static bool first_call = TRUE;
-
-	if (first_call) {
-		iterate_over_source_files(set_max_src_width, FALSE);
-		first_call = FALSE;
-	}
-/* RGA need to keep to char dimensions so overlay works */
-/*	sz->sz_depth = wn_get_sysfont()->ft_height + 1;  */
-	sz->sz_depth = wn_get_sysfont()->ft_height;
-/* RGA use an exact multiple of font width so display */
-/* overlay matches */
-	sz->sz_width = Max_src_width + wn_get_sysfont()->ft_width;
-/*	sz->sz_width = Max_src_width + OBJECT_HGAP;*/
+  srcfile_getsize_internal(unused_obj, unused_par, sz, FALSE);
 }
 
+static 
 void
-module_getsize(unused_obj, unused_par, sz)
+module_getsize_internal(unused_obj, unused_par, sz, reset)
 objid_t unused_obj, unused_par;
 struct szst *sz;
+bool reset;
 {
 	static bool first_call = TRUE;
 
+	if (reset)
+	{
+	  first_call = TRUE;
+	  return;
+	}
 	if (first_call) {
 		iterate_over_all_modules(set_max_module_width);
 		first_call = FALSE;
 	}
 
-/* RGA need to keep to char dimensions so overlay works */
-/*	sz->sz_depth = wn_get_sysfont()->ft_height + 1;*/
 	sz->sz_depth = wn_get_sysfont()->ft_height;
-/* RGA use an exact multiple of font width so display */
-/* overlay matches */
 	sz->sz_width = Max_src_width + wn_get_sysfont()->ft_width;
-/*	sz->sz_width = Max_module_width + OBJECT_HGAP;*/
+}		
+			
+void
+module_getsize(unused_obj, unused_par, sz)
+objid_t unused_obj, unused_par;
+struct szst *sz;
+{
+  module_getsize_internal(unused_obj, unused_par, sz, FALSE);
 }		
 			
 void
@@ -988,3 +1049,12 @@ long *p_fg, *p_bg;
   *p_fg = color.co_pixel;
   *p_bg = WN_BG(wn);
 }
+
+void
+reset_filesize_operations()
+{
+  srcfile_getsize_internal(NULL, NULL, NULL, TRUE); /* reset */
+  module_getsize_internal(NULL, NULL, NULL, TRUE); /* reset */
+  Max_src_width = 0;
+  Max_module_width = 0;
+}
