@@ -27,7 +27,7 @@ char *sig_format_obj PROTO((objid_t code));
 void do_sig PROTO((objid_t obj, int command, char *arg));
 void sig_getsize PROTO((objid_t obj, objid_t unused_par, sz_t *sz));
 const char *sig_getobjname PROTO((objid_t obj));
-void sig_getcolor PROTO((objid_t obj, long *p_fg, long *p_bg));
+void sig_getcolor PROTO((objid_t obj, int wn, long *p_fg, long *p_bg));
 
 extern const char Sghead_format[];
 extern const char Sig_format[];
