@@ -10,6 +10,7 @@
 char ups_obj_stack_c_sccsid[] = "@(#)obj_stack.c	1.21 20 Jun 1995 (UKC)";
 
 #include <mtrprog/ifdefs.h>
+#include "ao_ifdefs.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,6 +46,14 @@ char ups_obj_stack_c_sccsid[] = "@(#)obj_stack.c	1.21 20 Jun 1995 (UKC)";
 #include "debug.h"
 #include "tdr.h"
 #include "state.h"
+#include "exec.h"
+
+#ifdef AO_ELF
+#include "ao_elflib.h"
+#endif
+#if defined(AO_AOUT_SHLIBS)
+#include "ao_shlib.h"
+#endif
 
 /*  Arguments block for label_if_var_addr.
  */
@@ -221,9 +230,19 @@ static char *
 badfuncname(addr)
 taddr_t addr;
 {
-	char buf[50];
+	char buf[256];
+	char *unload_lib = NULL;
 
-	(void) sprintf(buf, "<bad text address 0x%lx>", addr);
+#ifdef AO_ELF
+	unload_lib = get_unloaded_lib(addr);
+#endif 
+#if defined(AO_AOUT_SHLIBS)
+	unload_lib = get_unloaded_lib(addr);
+#endif 
+	if (unload_lib)
+	  (void) sprintf(buf, "<%s>", unload_lib);
+	if (!unload_lib)
+	  (void) sprintf(buf, "<bad text address 0x%lx>", addr);
 	return strsave(buf);
 }
 
@@ -321,7 +340,8 @@ fval_t unused_arg;
 	int objtype; 
 
 	objtype = get_object_type(obj);
-	if (objtype != OT_FUNC && objtype != OT_BADFUNC) {
+	if (objtype != OT_FUNC && objtype != OT_BADFUNC &&
+	    objtype != OT_UNLOADED_LIB) {
 		remove_object(obj, OBJ_DESCENDENTS);
 		remove_object(obj, OBJ_SELF);
 	}
@@ -428,6 +448,7 @@ fval_t *fields;
 }
 
 const char Badfunc_format[] = "%[-]30cs\n";
+const char Nolibfunc_format[] = "%[-]60cs\n";
 #define FN_BADFUNC_NAME	0
 
 /*  Add function objects to the display tree.
@@ -528,10 +549,20 @@ objid_t par;
 			source_stk = stk;
 
 		if ((stk->stk_func->fu_flags & FU_BAD) != 0) {
-			change_type(newcode, OT_BADFUNC);
-			set_field_value(newcode, FN_BADFUNC_NAME,
-						badfuncname(stk->stk_pc));
-			free((char *)fields[FN_FUNC_LNUM]);
+		  char *unload_lib = NULL;
+#ifdef AO_ELF
+		  unload_lib = get_unloaded_lib(stk->stk_pc);
+#endif 
+#if defined(AO_AOUT_SHLIBS)
+		  unload_lib = get_unloaded_lib(stk->stk_pc);
+#endif 
+		  if (unload_lib)
+		    change_type(newcode, OT_UNLOADED_LIB);
+		  else
+		    change_type(newcode, OT_BADFUNC);
+		  set_field_value(newcode, FN_BADFUNC_NAME,
+				  badfuncname(stk->stk_pc));
+		  free((char *)fields[FN_FUNC_LNUM]);
 		}
 		else {
 			set_all_fields(newcode, fields, (fval_t)NULL);
@@ -733,6 +764,46 @@ char **p_arg;
 	return check_single_expr(command == MR_ADD_EXPRESSION);
 }
 
+void 
+post_do_nolibfunc (command, arg)
+int command;
+char *arg;
+{
+  if (can_get_target_vars(get_current_target()))
+    refresh_target_display(get_current_target(), SR_BPT, TRUE);
+}
+
+void
+do_nolibfunc(par, command, arg)
+objid_t par;
+int command;
+char *arg;
+{
+  char *str, *lib;
+  
+  switch(command) {
+  case MR_LOAD_LIBRARY:
+    str = badstack_format_obj(par);
+    lib = strsave(str + 1);	/* remove '<' */
+    lib[strlen(lib)-1] = 0;	/* remove '>' */
+    add_to_load_path(lib, 0, 1); /* load requested */
+#ifdef AO_TARGET
+#if defined(AO_AOUT_SHLIBS)
+    unload_shared_library_symtabs(get_current_target());
+    load_shared_library_symtabs(get_current_target(), TRUE);
+#endif /* AO_AOUT_SHLIBS */
+    if (can_get_target_vars(get_current_target()))
+      rescan_dynamic_solibs(get_current_target(), FALSE);
+    /*    load_from_solibs(get_current_target(), lib);*/
+#endif /* AO_TARGET */
+    free(lib);
+    break;
+    
+  default:
+    panic("bad cmd in dbf");
+  }
+}
+
 /*  Process the return from a function line menu.
  *  Add the variables to the display or conceal them.
  *  Tell the editor where we are.
@@ -1423,3 +1494,26 @@ display_var_in_mangled_form(buf, this_obj_exists, this_obj_var)
   free((char *)names);
   return (found || found2);
 }
+
+/*  Return the width in pixels of an uloaded lib object.
+ *
+ *  A pointer to this function is passed to the obj library.
+ */
+/* ARGSUSED */
+
+void
+nolibfunc_getsize(obj, unused_par, sz)
+objid_t obj, unused_par;
+struct szst *sz;
+{
+	char *str;
+	int width = 100;
+
+	str = badstack_format_obj(obj);
+	if (str)
+	  width = wn_strwidth(str, (font_t *)NULL);
+	sz->sz_depth = wn_get_sysfont()->ft_height;
+	sz->sz_width = width + wn_get_sysfont()->ft_width;
+	if (str)
+	  free(str);
+}
