@@ -41,6 +41,8 @@ const char *func_getobjname PROTO((objid_t code));
 char *func_format_obj PROTO((objid_t code));
 int pre_do_func PROTO((int command, char **p_arg));
 void do_func PROTO((objid_t par, int command, char *arg));
+void do_nolibfunc PROTO((objid_t par, int command, char *arg));
+void post_do_nolibfunc PROTO((int command, char *arg));
 char *block_format_obj PROTO((objid_t code));
 int pre_do_block PROTO((int command, char **p_arg));
 void do_block PROTO((objid_t obj, int command, char *arg));
@@ -82,6 +84,7 @@ int display_var_in_mangled_form PROTO((const char *buf, int *this_obj_exists,
 extern const char Func_format[];
 extern const char Fsig_format[];
 extern const char Badfunc_format[];
+extern const char Nolibfunc_format[];
 extern const char Block_format[];
 
 #ifdef OBJTYPES_H_INCLUDED
