@@ -152,6 +152,8 @@ char *textcopy;
 		fe->text = (textcopy != NULL) ? textcopy : (char *)dr->dr_fval;
 		fe->display = make_text_display(fe->wn, dr->dr_fg, dr->dr_bg,
 						fe->text);
+		if ( fm->edit_history)
+		    edit_set_history(fe->display, *fm->edit_history);
 		fe->quitfunc = fm->quitfunc;
 		fe->move_point_func = NULL;
 		fe->obj = obj;
@@ -225,6 +227,13 @@ char *handle;
 	edit_redraw_display(((Field_edit_info *)handle)->display);
 }
 
+Edit_display*
+field_edit_get_display(handle)
+char *handle;
+{
+	return ((Field_edit_info *)handle)->display;
+}
+
 void
 field_edit_handle_key_event(region, handle, ev)
 Region *region;
@@ -345,6 +354,7 @@ const char **p_display_string;
 		ok = (*fe->quitfunc)(fe->obj, text, p_display_string);
 
 		if (ok) {
+			edit_save_in_history(fe->display);
 			obj_update_size(fe->obj);
 		}
 		else {
