@@ -29,6 +29,21 @@ static Pathentry *Pathlist = NULL, *Pathlist_tail;
 
 static bool Use_path_hints = TRUE;
 
+void
+srcpath_free_pathlist()
+{
+  Pathentry *pl, *next;
+
+  for (pl = Pathlist; pl != NULL; pl = next)
+  {
+    next = pl->next;
+    free((char *)pl->path);
+    free(pl);
+  }
+  Pathlist = NULL;
+  Pathlist_tail = NULL;
+}
+
 Pathentry *
 get_source_path_root()
 {
