@@ -1319,7 +1319,7 @@ char *data;
 	return TRUE;
 }
 
-void
+int
 srcwin_set_point(sw, region, ev)
 Srcwin *sw;
 Region *region;
@@ -1343,12 +1343,14 @@ event_t *ev;
 	    eb = (Editblock *)edit_get_propchange_data(pc);
 
 	if (eb == NULL) {
-		errf("Not in an editable section");
-		wn_wait_for_release_of(ev->ev_wn, B_ANY);
+	  /*		errf("Not in an editable section");*/
+	  /*		wn_wait_for_release_of(ev->ev_wn, B_ANY);*/
+	  return 0;		/* RGA allow middle click to uses tags */
 	}
 	else {
 		srcwin_start_edit(sw, region, eb, point);
 	}
+	return 1;
 }
 
 void
@@ -1902,3 +1904,47 @@ void display_area_overlay_unselect_global_selection()
     Outwin *dw = (Outwin *)re_get_data(get_display_area_region());
     edit_unselect_global_selection(dw->display);
 }
+
+static 
+void
+srcwin_clear_stopline()
+{
+  Highlighting *hl = &Stopline;
+
+  hl->buffer = NULL;
+  hl->start = NULL;
+  hl->lim = NULL;
+}
+
+void 
+srcwin_reset()
+{
+  Srcwin *sw = get_current_srcwin();
+  Outwin *ow;
+  Text_buffer *tb;
+  Edit_buffer *buffer;
+  font_t *font;
+  int wn, width, height, nchars;
+  char nbuf[60];
+
+  srcwin_clear_stopline();
+  sw->fil = NULL;
+  font = wn_get_sysfont();
+  wn = (int)srcwin_get_data(sw);
+  wn_get_window_size(wn, &width, &height);
+  nchars = (width - 8) / font->ft_width;
+  if (nchars > sizeof(nbuf) - 1)
+    nchars = sizeof(nbuf) - 1;
+  memset(nbuf, ' ', 60);
+  nbuf[nchars] = 0;
+  wn_tputs(wn, nbuf, 4, (height - font->ft_height) / 2);
+
+  ow = &sw->outwin;
+  tb = text_create_empty_lbuf_buffer(alloc_create_pool(), 0, 1024);
+  buffer = edit_create_buffer(tb, Srcfont_info, 0);
+  if (buffer != NULL)
+    edit_set_buffer(ow->display, buffer);
+  ow->buffer = buffer;
+  srcbuf_reset_file_handle();
+  srcwin_redraw(sw);
+}
