@@ -27,7 +27,6 @@ char ups_st_util_c_sccsid[] = "@(#)st_util.c	1.5 04 Jun 1995 (UKC)";
 
 #include <local/ukcprog.h>
 #include <local/arg.h>
-#include <mtrprog/genmergesort.h>
 #include <mtrprog/strcache.h>
 #include <mtrprog/alloc.h>
 
@@ -42,6 +41,7 @@ char ups_st_util_c_sccsid[] = "@(#)st_util.c	1.5 04 Jun 1995 (UKC)";
 #include "srcwin.h"
 #include "srcpath.h"
 #include "ui.h"
+#include "tdr.h"
 
 /*  Element in the linked list of global variable names and 
  *  addresses for a symbol table.
@@ -60,63 +60,193 @@ struct addrlist_s {
 typedef struct loadlist_s {
 	const char *ld_load;
 	int str_style;
+	int load_requested;
 	struct loadlist_s *ld_next;
 } loadlist_t;
 
-static loadlist_t *Loadlist = NULL, *Loadlist_tail;
-static int Noload_flag = 0;
+typedef struct noloadlist_s {
+	const char *name;
+	int loaded;
+	struct noloadlist_s *ld_next;
+} noloadlist_t;
 
-static block_t *make_rootblock PROTO((void));
+typedef struct name_s{
+  const char *name;
+} name_t;
 
 #ifdef AO_ELF
 bool elf_scan_dot_o_file_symtab PROTO((fil_t *fil));
 #endif /* !AO_ELF */
 
-void
-free_library_load_list()
-{
-  loadlist_t *ld, *ld1;
+static noloadlist_t *NoLoadlist = NULL, *NoLoadlist_tail;
+static block_t *make_rootblock PROTO((void));
+static loadlist_t *Loadlist = NULL, *Loadlist_tail;
+static int Noload_flag = 0;
 
-  for (ld = Loadlist; ld != NULL; )
+void
+free_library_noload_list()
+{
+  noloadlist_t *ld, *ld1;
+
+  for (ld = NoLoadlist; ld != NULL; )
   {
     ld1 = ld->ld_next;
+    free((char *)ld->name);
     free(ld);
     ld = ld1;
   }
-  Loadlist = NULL;
+  NoLoadlist = NULL;
 }
 
 void
-add_to_load_path(path, noload)
+add_to_noload_list(name, loaded)
+const char *name;
+int loaded;
+{
+        noloadlist_t *ld;
+
+
+	/* adding a new entry, so check if it's already in the list */
+	for (ld = NoLoadlist; ld != NULL; ld = ld->ld_next)
+	  if (!ld->loaded && ld->name && !strcmp(ld->name, name))
+	  {
+	    ld->loaded = loaded;
+	    return;
+	  }
+
+        ld = (noloadlist_t *)e_malloc(sizeof(noloadlist_t));
+	ld->name = strsave(name);
+	ld->loaded = loaded;
+        ld->ld_next = NULL;
+
+        if (NoLoadlist == NULL)
+                NoLoadlist = ld;
+        else
+                NoLoadlist_tail->ld_next = ld;
+        NoLoadlist_tail = ld;
+}
+
+static int
+ncmp(name1, name2)
+name_t *name1, *name2;
+{
+  if (!name1 || !name2)
+    return 1;
+  return strcmp(name1->name,  name2->name);
+}
+
+int
+get_noload_list(names)
+const char ***names;
+{
+        noloadlist_t *ld;
+	const char **ptr;
+	int i;
+	name_t *n, *nroot;
+
+	for (i = 0, ld = NoLoadlist; ld != NULL; ld = ld->ld_next)
+	  if (!ld->loaded && ld->name)
+	    i++;
+	if (i == 0)
+	  return 0;
+	nroot = n = malloc(sizeof(name_t*)*(i+1));
+	for (ld = NoLoadlist; ld != NULL; ld = ld->ld_next)
+	  if (!ld->loaded && ld->name)
+	  {
+	    n->name = ld->name;
+	    n++;
+	  }
+	n->name = 0;
+	qsort((char *)nroot, i, sizeof(name_t *), ncmp);
+	*(names) = ptr = malloc(sizeof(const char **)*(i+1));
+	for (n = nroot; n->name && ptr; n++, ptr++)
+	  *ptr = n->name;
+	*ptr = 0;
+	free(nroot);
+	return 1;
+}
+
+void
+free_library_load_list()
+{
+  loadlist_t *ld, *ld1, **tail_ld_ptr, **last_ld_ptr;
+
+  for (last_ld_ptr = &Loadlist, tail_ld_ptr = &Loadlist_tail,
+	 Loadlist_tail = 0, ld = Loadlist; ld != NULL; )
+  {
+    ld1 = ld->ld_next;
+    if (!ld->load_requested)
+    {
+      *last_ld_ptr = ld->ld_next;
+      free((char*)ld->ld_load);
+      free(ld);
+    }
+    else	/* keep these */
+    {
+      *tail_ld_ptr = *last_ld_ptr = ld;
+      last_ld_ptr = &ld->ld_next;
+    }
+    ld = ld1;
+  }
+}
+
+void
+add_to_load_path(path, noload, load_requested)
 const char *path;
 int noload;
+int load_requested;
 {
         loadlist_t *ld;
 	char *buf;
+	const char *path_ptr = path;
+
+	if (load_requested)
+	{
+	  for (ld = Loadlist; ld != NULL; ld = ld->ld_next)
+	    if (strstr(path_ptr, ld->ld_load))
+	    {
+	      ld->load_requested = 1;
+	      return;
+	    }
+	}
 
-	buf = strsave(path);
         ld = (loadlist_t *)e_malloc(sizeof(loadlist_t));
-	if (*path == '*')
+	ld->load_requested = load_requested;
+	if (*path_ptr == '.' && *(path_ptr+1) == '/')
+	{
+	  const char *source_path;
+
+	  source_path = hold_default_source_path(NULL, FALSE); /* get */
+	  path_ptr = (const char *)e_malloc(strlen(source_path) + strlen(path));
+	  sprintf((char *)path_ptr, "%s%s", source_path, path+1);
+	}
+
+	buf = strsave(path_ptr);
+
+	if (*path_ptr == '*')
 	{
 	  ld->str_style = LD_STRSTR;
-	  if (path[strlen(path)-1] == '*')
-	  buf[strlen(path)-1] = 0;
+	  if (path_ptr[strlen(path_ptr)-1] == '*')
+	  buf[strlen(path_ptr)-1] = 0;
 	  ld->ld_load = strsave(buf + 1);
 	}
 	else
-	  if (path[strlen(path)-1]  == '*')
+	  if (path_ptr[strlen(path_ptr)-1]  == '*')
 	  {
 	    ld->str_style = LD_STRNCMP;
-	    buf[strlen(path)-1] = 0;
+	    buf[strlen(path_ptr)-1] = 0;
 	    ld->ld_load = strsave(buf);
 	  }
 	  else
 	  {
 	    ld->str_style = LD_STRCMP;
-	    ld->ld_load = strsave(path);
+	    ld->ld_load = strsave(path_ptr);
 	  }
 	free(buf);
-	Noload_flag = noload;
+	if (path_ptr != path)
+	  free((char *)path_ptr);
+	if (!load_requested)
+	  Noload_flag = noload;
         ld->ld_next = NULL;
 
         if (Loadlist == NULL)
@@ -141,15 +271,15 @@ int user_wants_library_loaded(name)
     {
     case LD_STRCMP:
       if (!strcmp(name, ld->ld_load))
-	return(Noload_flag ? 0 : 1);
+	return(ld->load_requested ? 1 : Noload_flag ? 0 : 1);
       break;
     case LD_STRNCMP:
       if (!strncmp(name, ld->ld_load, strlen(ld->ld_load)))
-	return(Noload_flag ? 0 : 1);
+	return(ld->load_requested ? 1 : Noload_flag ? 0 : 1);
       break;
     case LD_STRSTR:
       if (strstr(name, ld->ld_load))
-	return(Noload_flag ? 0 : 1);
+	return(ld->load_requested ? 1 : Noload_flag ? 0 : 1);
       break;
     }
   }
@@ -304,6 +434,7 @@ char *data;
 	st->st_ops = ops;
 	st->st_modtime = 0;
 	st->st_eclipsed = 0;
+	st->st_func_hv = 0;
 
 	for (f = funclist; f != NULL; f = f->fu_next)
 		f->fu_symtab = st;
@@ -344,6 +475,8 @@ symtab_t *st;
 	free_cblist_info(st->st_cblist);
 
 	st_close_symtab_data(st);
+	if (st->st_func_hv)
+	  free(st->st_func_hv);
 
 	/*  Free all storage associated with this symtab.
 	 *  Vars, functions, common blocks, the lot.
@@ -645,9 +778,9 @@ bool rematch, reload;
 
     if (rematch || reload)
     {
-      if ((fp = fopen(get_temp_bpt_filename(), "w")) == NULL)
+      if ((fp = fopen(get_temp_state_filename(), "w")) == NULL)
 	errf("Can't create temp saved breakpoints file `%s'",
-	     get_temp_bpt_filename());
+	     get_temp_state_filename());
       else
       {
 	save_matching_breakpoints(NULL, fil, (char *)fp);
@@ -720,6 +853,8 @@ bool rematch, reload;
 	    {
 	      errf("\bUsing %s", nbuf);
 	      fil->fi_source_path = path_used;
+	      if (fil->fi_srcbuf)
+		srcbuf_destroy(fil->fi_srcbuf);
 	      fil->fi_srcbuf = srcbuf; 
 	      fil->fi_flags |= FI_NEEDS_RESCAN;
 	      break;
@@ -733,7 +868,11 @@ bool rematch, reload;
 	    }
 	  }
 	  else
+	  {
+	    if (srcbuf)
+	      srcbuf_destroy(srcbuf);
 	    break;
+	  }
 	}
       }
     if (pl == NULL)
@@ -743,6 +882,8 @@ bool rematch, reload;
 	(void) sprintf(nbuf, "%s%s", opened_pl_path, fil->fi_name);
 	errf("\bUsing %s", nbuf);
 	fil->fi_source_path = opened_pl_path;
+	if (fil->fi_srcbuf)
+	  srcbuf_destroy(fil->fi_srcbuf);
 	fil->fi_srcbuf = opened_srcbuf; /* use first one that opened */
 	fil->fi_flags |= FI_NEEDS_RESCAN;
       }
@@ -764,6 +905,8 @@ bool rematch, reload;
     if (fil->fi_language != LANG_CC && reload && fil->fi_srcbuf)
     {
       fil->fi_flags |= FI_NEEDS_RESCAN;
+      if (fil->fi_srcbuf)
+	srcbuf_destroy(fil->fi_srcbuf);
       fil->fi_srcbuf = srcpath_visit_file(fil->fi_symtab->st_apool,
 					  fil->fi_path_hint,
 					  fil->fi_name);
@@ -771,7 +914,7 @@ bool rematch, reload;
     fil->fi_flags |= FI_DONE_MATCH;
     if (rematch || reload)
       if (fp != NULL)
-	restore_cached_bpts();
+	restore_target_state(FALSE, TRUE);
   }
   /* RGA end */		
 	
@@ -866,7 +1009,7 @@ symtab_t *fu_symtab;
 void
 restore_cached_symtab_breakpoints()
 {
-  restore_cached_bpts();
+  restore_target_state(FALSE, TRUE);
 }
 
 void
@@ -896,11 +1039,11 @@ fil_t *fil;
 }
 
 char *
-get_temp_bpt_filename()
+get_temp_state_filename()
 {
   static char filename[32];
 
   if (!*filename)
-    sprintf(filename, "/tmp/ups-bpts.%d", (int)getpid());
+    sprintf(filename, "/tmp/ups-state.%d", (int)getpid());
   return filename;
 }
