@@ -128,6 +128,8 @@ typedef struct {
 	bool xp_new_dynamic_libs_loaded; /* RGA set if dynamic libs loaded */
 	bool xp_hit_solib_event;	/* RGA if bpt in dynamic linker hit */
 	bool xp_restore_bpts;		/* RGA restore old bpts after attach */
+	bool xp_target_updated;   /* RGA reread main symtab */
+	bool xp_target_name_changed;    /* RGA when target name has been edited */
 } target_t;
 
 typedef union text_block_s {
@@ -234,7 +236,7 @@ typedef struct xp_ops_s {
 
 	/*  Miscellaneous.
 	 */
-	int (*xo_write_corefile)PROTO((target_t *xp, const char *name));
+	int (*xo_load_library)PROTO((target_t *xp, char *arg));
 
 	int (*xo_call_func)PROTO((target_t *xp, char *cma, taddr_t addr,
 				  taddr_t *args, size_t *argsizes, int nargs,
@@ -328,8 +330,8 @@ typedef struct xp_ops_s {
 #define xp_read_text_from_process(xp, addr, buf, nbytes) \
 	(xp->xp_ops->xo_read_text_from_process)(xp, addr, buf, nbytes)
 
-#define xp_write_corefile(xp, name) \
-	(xp->xp_ops->xo_write_corefile)(xp, name)
+#define xp_load_library(xp, arg) \
+	(xp->xp_ops->xo_load_library)(xp, arg)
 #define xp_call_func(xp, ma, addr, args, argsizes, nargs, p_res, p_mesg) \
 	(xp->xp_ops->xo_call_func)(xp, ma, addr, args, argsizes, nargs, \
 								  p_res, p_mesg)
@@ -345,10 +347,11 @@ bool can_get_target_vars PROTO((target_t *xp));
 bool target_process_exists PROTO((target_t *xp));
 void kill_or_detach_from_target PROTO((target_t *xp));
 stopres_t xp_execto PROTO((target_t *xp, taddr_t addr));
-void write_target_core PROTO((target_t *xp));
+void load_one_library PROTO((target_t *xp, char *arg));
 int make_target PROTO((const char *textpath, const char *corename,
 		       bool user_gave_core, target_t **p_xp,
-		       const char **p_cmdline, bool restore_bpts));
+		       const char **p_cmdline, bool restore_bpts,
+		       bool target_updated));
 
 taddr_t xp_getreg PROTO((target_t *xp, int regno));
 
@@ -380,5 +383,9 @@ bool extract_bool_arg PROTO((ebuf_t *usage_eb, char **argv,
 xp_ops_t **get_target_drivers PROTO((void));
 void target_stop PROTO((void));
 int user_wants_non_frame_functions PROTO((void));
-int attach_to_target PROTO((target_t *xp, char *pid_str, bool first_attach));
-void rescan_dynamic_solibs PROTO((target_t *xp));
+int attach_to_target PROTO((target_t *xp, char *pid_str, bool first_attach,
+			    bool target_updated, bool user_gave_core,
+			    bool new_target_name));
+void rescan_dynamic_solibs PROTO((target_t *xp, bool check_only));
+int check_target PROTO((const char *textpath, int *p_index, int *p_fd,
+			long *p_mtime, bool check_textfile));
