@@ -292,6 +292,7 @@ static actmap_t Am_var[] = {
 	{ MR_VAR_SHOW_HIERARCHY,	"show_hierarchy"		},
 	{ MR_VAR_SHOW_STATICS,		"show_statics"			},
 	{ MR_VAR_DUMP_TO_OUTWIN,	"dump_to_outwin"		},
+	{ MR_VAR_SHOW_SIZE,		"show_size"			},
 };
 
 static actmap_t Am_srcwin[] = {
@@ -416,7 +417,7 @@ static actmap_t Am_pmenu[] = {
 };
 
 static actmap_t Am_target[] = {
-	{ MR_WRITE_CORE,		"write_core"			},
+	{ MR_LOAD_ONE_LIBRARY,		"load_library"			},
 	{ MR_SAVE_STATE,		"save_state"			},
 	{ MR_LOAD_STATE,		"load_state"			},
 	{ MR_RESCAN_INIT_FILE,		"rescan_init_file"		},
@@ -1834,7 +1835,7 @@ char **args;
 		return -1;
 	}
 	load_flag = TRUE;
-	add_to_load_path(*args, 0); /* load */
+	add_to_load_path(*args, 0, 0); /* load */
 	return 0;
 }
 
@@ -1855,7 +1856,7 @@ char **args;
 		return -1;
 	}
 	noload_flag = TRUE;
-	add_to_load_path(*args, 1); /* noload */
+	add_to_load_path(*args, 1, 0); /* noload */
 	return 0;
 }
 
@@ -2373,7 +2374,7 @@ char **args;
 	}
 
 	sw = Have_window ? get_current_srcwin() : NULL;
-	show_var(sw, fil, lnum, varname, 0);
+	show_var(sw, fil, lnum, varname, 0, 0);
 
 	td_record_show_var(fil, lnum, varname);
 	
@@ -2951,3 +2952,20 @@ const char *filename;
 
 	return 0;
 }
+
+void
+rescan_init_file()
+{
+  free_library_load_list();
+  if (td_init_from())
+  {
+    bool eof;
+    int res;
+    
+    res = td_load_loop(&eof);
+    
+    if (res != 0)
+      fprintf(stderr, "Error in init file\n");
+    td_restore_replay_fp();
+  }
+}
