@@ -35,6 +35,7 @@ char ups_trun_j_c_sccsid[] = "@(#)trun_j.c	1.32 24 May 1995 (UKC)";
 #include "obj_bpt.h"
 #include "srcbuf.h"
 #include "dx.h"
+#include "expr.h"
 
 #define MAXVARLEN	128
 
@@ -76,6 +77,7 @@ continue_target(xp, p_stopres)
 target_t *xp;
 stopres_t *p_stopres;
 {
+	static int recursive=-1;
 	stopres_t stopres;
 	int stop, sig;
 	breakpoint_t *bp;
@@ -86,6 +88,7 @@ stopres_t *p_stopres;
 	if (target_process_exists(xp) && install_all_breakpoints(xp) != 0)
 		return;
 
+	recursive++;
 	do {
 		if (target_process_exists(xp)) {
 			stopres = xp_restart_child(xp, get_restart_sig(xp),
@@ -124,11 +127,19 @@ stopres_t *p_stopres;
 			break;
 		case SR_BPT:
 			bp = get_breakpoint_at_addr(xp, xp_getreg(xp, UPSREG_PC));
-			stop = execute_bp_code(bp, xp_getreg(xp, UPSREG_FP),
-						   xp_getreg(xp, UPSREG_AP));
-			stopres = xp_get_stopres(xp);
-			if (stopres != SR_BPT)
-				stop = TRUE;
+			/* RCB: Don't execute breakpoint code if recursive;
+			 *      let the top level call do it.
+			 */
+			if ( recursive )
+			    stop = TRUE;
+			else
+			{
+			    stop = execute_bp_code(bp, xp_getreg(xp, UPSREG_FP),
+						       xp_getreg(xp, UPSREG_AP));
+			    stopres = xp_get_stopres(xp);
+			    if (stopres != SR_BPT)
+				    stop = TRUE;
+			}
 			break;
 		default:
 			panic("stopres botch in ct");
@@ -137,6 +148,7 @@ stopres_t *p_stopres;
 	} while (!stop);
 	
 	*p_stopres = stopres;
+	recursive--;
 }
 
 /*  Return TRUE if a jump is needed.  The only jumps that aren't needed
@@ -1230,7 +1242,6 @@ target_t *xp;
 rtype_t rtype;
 {
 	stopres_t stopres;
-	char *def;
 	static int non_frame_functions = -1;
 
 	if (non_frame_functions == -1)
