@@ -124,4 +124,10 @@ int rational_products_running PROTO((void));
 
 int  get_run_alarm_time PROTO((void));
 void run_alarm_time_expired PROTO((void));
+char *get_target_name PROTO((void));
 
+void run_done PROTO((int suggest_raise));
+void reset_pathlist PROTO((char *textpath));
+void update_icon_name PROTO((char *textname));
+const char *hold_default_source_path PROTO((const char *path, bool set));
+void load_library_input PROTO((void *handle, char *arg));
