@@ -10,6 +10,7 @@
 char ups_ui_layout_c_sccsid[] = "@(#)ui_layout.c	1.32 20 Jun 1995 (UKC)";
 
 #include <mtrprog/ifdefs.h>
+#include "ao_ifdefs.h"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -53,6 +54,11 @@ char ups_ui_layout_c_sccsid[] = "@(#)ui_layout.c	1.32 20 Jun 1995 (UKC)";
 #include "va.h"
 #include "sccsdata.h"
 #include "util.h"
+#include "st.h"
+#include "target.h"
+#ifdef AO_ELF
+#include "ao_elflib.h"
+#endif
 
 Edit_history* typing_line_history = NULL;
 
@@ -1116,6 +1122,59 @@ static unsigned char enter_image_bits[] = {
 			wn, win_x, win_y, R_RPL);
 }
 
+void
+load_library_input(handle, arg)
+void *handle;
+char *arg;
+{
+  event_t *ev;
+  popup_t loadlibrarypop;
+  const char **loadlibrary;
+  int res, buttons;
+  Region* focus_re = re_get_keyboard_focus_region();
+
+  ev = (event_t *)arg;
+  buttons = ev->ev_buttons;
+  res = get_noload_list(&loadlibrary);
+  if (res == 0)
+  {
+    wn_wait_for_release_of(ev->ev_wn, B_ANY);
+    errf("\bAll libraries loaded");
+  }
+  else
+  {
+    loadlibrarypop.po_mdesc = -1;
+    loadlibrarypop.po_save_last = FALSE;
+    loadlibrarypop.po_last = 0;
+    loadlibrarypop.po_caps = loadlibrary;
+    loadlibrarypop.po_left = TRUE;
+  
+    res = select_from_popup(ev->ev_wn, "UnloadedLibMenu", buttons, &loadlibrarypop,
+			    ev->ev_x, ev->ev_y);
+    Mclose(loadlibrarypop.po_mdesc);
+    if ( res >= 0 )
+    {
+      add_to_load_path(*(loadlibrary+res), 0, 1); /* load requested */
+#ifdef AO_ELF
+      if (can_get_target_vars((target_t *)handle))
+      {
+	rescan_dynamic_solibs((target_t *)handle, FALSE);
+	refresh_target_display((target_t *)handle, SR_BPT, TRUE);
+      }
+      add_to_noload_list(*(loadlibrary+res), 1); /* mark as loaded */
+#endif
+#if defined(AO_AOUT_SHLIBS)
+      if (can_get_target_vars((target_t *)handle))
+	rescan_dynamic_solibs((target_t *)handle, FALSE);
+      unload_shared_library_symtabs((target_t *)handle);
+      load_shared_library_symtabs((target_t *)handle, TRUE);
+      refresh_target_display((target_t *)handle, SR_BPT, TRUE);
+#endif
+    }
+    free(loadlibrary);
+  }
+}
+
 static void
 line_draw(region, wn, width, height, draw_reason)
 Region *region;
@@ -1634,7 +1693,7 @@ int split;
 					srcwin_release_focus);
 	
 	re_set_cursor(regs[TEXT], CU_DISPLAY);
-	re_set_mhcaps(regs[TEXT], "select", "edit", "menu");
+	re_set_mhcaps(regs[TEXT], "select", "edit/goto", "menu");
 
 	re_set_callbacks(regs[NAMEWIN], (re_draw_proc_t)NULL,
 			 name_window_event_handler, (re_destroy_proc_t)NULL);
@@ -2385,16 +2444,21 @@ event_t *ev;
 
 	wn = ev->ev_wn;
 	mask = ~(1 << ma->ma_md);
-	if (!Mselect(ev->ev_x, ev->ev_y, wn, MS_PRESSED, mask))
-		return;
-
-	for (;;) {
-		wn_next_event(wn, EVENT_MASK, ev);
-		if ((ev->ev_buttons & B_LEFT) == 0)
-			break;
-		Mselect(ev->ev_x, ev->ev_y, wn, MS_CONTINUE, mask);
+	rv = Mselect(ev->ev_x, ev->ev_y, wn, MS_PRESSED, mask);
+	if (!rv)
+	  return;
+	if (rv == (int)MR_LOAD_ONE_LIBRARY && !strcmp(ma->ma_menu_name, "target"))
+	    ma->ma_data = (char *)ev;
+	else
+	{
+	  for (;;) {
+	    wn_next_event(wn, EVENT_MASK, ev);
+	    if ((ev->ev_buttons & B_LEFT) == 0)
+	      break;
+	    Mselect(ev->ev_x, ev->ev_y, wn, MS_CONTINUE, mask);
+	  }
+	  rv = MS_rv(Mselect(ev->ev_x, ev->ev_y, wn, MS_RELEASED, mask));
 	}
-	rv = MS_rv(Mselect(ev->ev_x, ev->ev_y, wn, MS_RELEASED, mask));
 
 	if (rv != 0) {
 
@@ -2403,6 +2467,8 @@ event_t *ev;
 
 	  if (step_menu || !strcmp(ma->ma_menu_name, "pmenu"))	
 	    re_expose(region, FALSE);
+	  if (!strcmp(ma->ma_menu_name, "target"))
+	    ma->ma_data = (char *)ev;
 	  td_record_menu_command(ma->ma_menu_name, rv);
 	  if (ma->ma_menu_func != permanent_menu_func ||
 	      !(rv == MR_DISWIN_SEARCH_FORWARDS ||
@@ -2554,19 +2620,3 @@ bool meta;
     evbuf.ev_buttons = 0;
   edit_handle_wn_key_event (display, &evbuf);
 }
-
-void
-redo_displayed_source_files()
-{
-  struct objst *obj, *ob;
-
-  obj = code_to_obj(path_to_obj("srchead"));
-  for (ob = obj->ob_child; ob; ob = ob->ob_next)
-    if (!ob->ob_child)
-      break;
-  if (!(!obj->ob_child || !ob))
-  {				/* source files displayed */
-    do_srchead((objid_t)obj->ob_code, MR_HIDE_SOURCE_FILES, (char *)NULL);
-    do_srchead((objid_t)obj->ob_code, MR_SHOW_SOURCE_FILES, (char *)NULL);
-  }
-}
