@@ -500,7 +500,11 @@ bool want_text;
 			}
 		}
 		
-		wn_next_event(wn, EV_BUTTON_UP | EV_MOUSE_MOVED, ev);
+		/* RGA added EV_WINDOW_EXPOSED for CDE with 'click to focus' and 
+		   'raise on focus' set. Then a click raises ups but it did not 
+		   refresh itself because the expose events were not requested */
+		wn_next_event(wn, EV_BUTTON_UP | EV_MOUSE_MOVED | EV_WINDOW_EXPOSED, 
+			      ev);
 
 	} while (ev->ev_type != EV_BUTTON_UP);
 
