@@ -56,7 +56,7 @@ struct Repltab {
 };
 
 static void do_var_selection PROTO((Srcwin *sw, event_t *ev, size_t sel_point,
-				    fil_t *fil, int lnum));
+				    fil_t *fil, int lnum, int skip_locals));
 static char *do_var_or_expr_selection PROTO((Edit_display *display,
 					     event_t *ev, size_t sel_point,
 					     size_t start, size_t lim,
@@ -579,12 +579,13 @@ bool *p_done_sel_loop;
 }
 
 static void
-do_var_selection(sw, ev, sel_point, fil, lnum)
+do_var_selection(sw, ev, sel_point, fil, lnum, skip_locals)
 Srcwin *sw;
 event_t *ev;
 size_t sel_point;
 fil_t *fil;
 int lnum;
+int skip_locals;
 {
         /* RGA add ' ' so we can click on 'a -> b" */
 	/* RCB: add '#' to check for "#stop" */
@@ -595,8 +596,7 @@ int lnum;
 	size_t start, lim;
 	bool done_sel_loop;
 	char *text;
-	int found = 0, *this_obj_var,
-	this_obj_exists = 0, mangled = 0;
+	int found = 0, *this_obj_var, this_obj_exists = 0, mangled = 0;
 	errf_ofunc_t oldf = NULL;
 	language_t fi_language;
 
@@ -676,7 +676,7 @@ int lnum;
 	      found = display_var_in_mangled_form
 		(text, &this_obj_exists, &this_obj_var);
 	    if (!found)
-	      found = show_var(sw, fil, lnum, text, oldf);
+	      found = show_var(sw, fil, lnum, text, oldf, skip_locals);
 			
 	    if (mangled)
 	      errf_set_ofunc(oldf);
@@ -696,7 +696,6 @@ int lnum;
 		/* inherited match */
 		errf("No function or variable `%s'", text);
 
-	      /*			show_var(sw, fil, lnum, text, 0);*/
 	    }
 
 	    free(text);
@@ -993,7 +992,7 @@ event_t *ev;
 {
 	Srcwin *sw;
 	Srcinfo srcinfo;
-	int buttons;
+	int buttons, skip_locals, set_point = 1;
 	
 	sw = (Srcwin *)re_get_data(region);
 
@@ -1010,13 +1009,20 @@ event_t *ev;
 
 	buttons = ev->ev_buttons & (B_ANY | B_SHIFT_MASK | B_CONTROL_MASK | B_MULT_CLICKS);
 
-	if (buttons == B_LEFT &&  srcinfo.have_point) {
+	if (buttons == B_MIDDLE) {
+		set_point = srcwin_set_point(sw, region, ev);
+	}
+
+	if ((!set_point ||	/* RGA allow middle to use tags */
+	     buttons == B_LEFT) &&
+	    srcinfo.have_point) {
 	    srcwin_have_editblock(sw, region, ev);
 	    indicate_target_menu_search(1);
 	    target_menu_search_disabled(1, 0); /* set */
 	    wn_clr_xclipboard();
+	    skip_locals = !!(ev->ev_buttons & B_CONTROL_MASK) || !set_point;
 	    do_var_selection(sw, ev, srcinfo.point,
-			     srcinfo.fil, srcinfo.fil_lnum);
+			     srcinfo.fil, srcinfo.fil_lnum, skip_locals);
 	    target_menu_search_disabled(0, 1); /* reset */
 	    indicate_target_menu_search(0);
 	}
@@ -1034,9 +1040,6 @@ event_t *ev;
 	if ((buttons & B_LEFT) && (buttons&B_MULT_CLICKS) &&  srcinfo.have_point) {
 	    srcwin_have_editblock(sw, region, ev);
 	}
-	if (buttons == B_MIDDLE) {
-		srcwin_set_point(sw, region, ev);
-	}
 	else if (ev->ev_type == EV_BUTTON_DOWN &&
 		 (buttons & ~B_SHIFT_MASK) == B_RIGHT &&
 		 srcinfo.fil_lnum != 0)
