@@ -67,8 +67,7 @@ char ups_ups_c_sccsid[] = "@(#)ups.c	1.53 04 Jun 1995 (UKC)";
 static void catch_fatal_sig PROTO((int signo));
 static void add_multiple_paths PROTO((const char *ro_paths, const char *defpath));
 static int is_directory PROTO((const char *name));
-static void get_default_paths PROTO((const char *textname,
-				     const char **p_dsrc, const char **p_dcore));
+static void get_default_paths PROTO((const char *textname, const char **p_dsrc, const char **p_dcore));
 static void usage PROTO((const char *mesg));
 static int setup_display PROTO((void));
 static int ups PROTO((target_t *xp, const char* textname, const char *args, bool replay,
@@ -147,7 +146,6 @@ const char *s;
 	Done_panic = TRUE;
 }
 
-
 /*  This routine is called when we get a signal that causes a core dump.
  *  We want to avoid overwriting an existing core file in the current
  *  directory because that may be from the target and thus precious to
@@ -297,6 +295,18 @@ const char *name;
 	return stat(name, &stbuf) == 0 && S_ISDIR(stbuf.st_mode);
 }
 
+const char *
+hold_default_source_path(path, set)
+const char *path;
+bool set;
+{
+  static const char *source_path = NULL;
+
+  if (set)
+    source_path = path;
+  return source_path;
+}
+
 static void
 get_default_paths(textname, p_dsrc, p_dcore)
 const char *textname;
@@ -537,7 +547,7 @@ Usage: ups -V\n\
 	    
 	
 	get_default_paths(textname, &default_source_path, &default_corename);
-
+	hold_default_source_path(default_source_path, TRUE); /* store */
 	got_source_path = FALSE;
 /*	want_startup_message = TRUE;*/
 /* RGA SLB only */
@@ -772,7 +782,7 @@ Usage: ups -V\n\
 	edit_initialize_history(state_dir, "editHistory",".upsEditHistory", history_items);
 
 	if (make_target(textname, corename, user_gave_core, &xp,
-			&args, FALSE) == -1)
+			&args, FALSE, FALSE) == -1)
 		exit(1);
 	set_current_target(xp);
 
@@ -1214,7 +1224,7 @@ bool attach;
 	xp = get_current_target(); /* RGA: attach may have changed xp */
 	if (res == 0 && want_save_config) {
 		if (target_process_exists(xp))
-			save_var_display_state();
+			save_var_display_state(TRUE, FALSE);
 		
 		res = save_state(state_dir, state_path, want_save_sigs);
 	}
@@ -1224,9 +1234,9 @@ bool attach;
 	if (target_process_exists(xp))
 		kill_or_detach_from_target(xp);
 
-	if ((file_access = access(get_temp_bpt_filename(), O_RDONLY)) != -1)
-	  if (unlink(get_temp_bpt_filename()) != 0)
-	    errf("Can't unlink `%s'", get_temp_bpt_filename());
+	if ((file_access = access(get_temp_state_filename(), O_RDONLY)) != -1)
+	  if (unlink(get_temp_state_filename()) != 0)
+	    errf("Can't unlink `%s'", get_temp_state_filename());
 
 	return res;
 }
@@ -1314,3 +1324,32 @@ rational_products_running()
     return rational_products;
 }
 
+/* Called when attaching/running a different target */
+void
+reset_pathlist(textpath)
+char *textpath;
+{
+  const char *default_source_path, *default_corename;
+
+  srcpath_free_pathlist();
+  get_default_paths(textpath, &default_source_path, &default_corename);
+  hold_default_source_path(default_source_path, TRUE); /* store */
+  add_multiple_paths(":", default_source_path);
+}
+
+/* Called when attaching/running a different target */
+void
+update_icon_name(textname)
+char *textname;
+{
+  int wn;
+  char title[20];
+
+  strcpy(title, "%a ");
+  if (strchr(textname, '/'))
+    textname = strrchr(textname,'/')+1;
+  strncpy(title+3,textname,sizeof(title)-4);
+  title[sizeof(title)-1] = 0;
+  wn = wn_get_root_window(get_message_wn());
+  wn_set_window_hints(wn, title,title, &ups_icon);
+}
