@@ -29,6 +29,7 @@ char ups_va_edit_c_sccsid[] = "@(#)va_edit.c	1.31 04 Jun 1995 (UKC)";
 #include "st.h"
 #include "obj_target.h"
 #include "obj_util.h"
+#include "obj_misc.h"
 #include "state.h"
 #include "va.h"
 #include "ci.h"
@@ -1638,3 +1639,63 @@ var_t *v;
 			return (objid_t)dv;
 	return NULL;
 }
+
+void show_dv_size(dv)
+     dvar_t* dv;
+{
+  type_t *type, *btype;
+  var_t *v;
+  int len;
+
+  v = dv->dv_var;
+  type = get_type_at_level(v, dv->dv_ilevel);
+  
+  len = UNKNOWN_SIZE;
+  /*  If the variable is an array then we can find out how
+		 *  many bytes are safe to store.
+		 */
+  if (dv->dv_ilevel > 0)
+  {
+    btype = get_type_at_level(v, dv->dv_ilevel - 1);
+
+    if (btype->ty_code == DT_ARRAY_OF) 
+      len = typesize(btype) - index_offset(dv);
+    else
+      len = typesize(btype);
+    if (len < 0)
+      len = UNKNOWN_SIZE;
+  }
+  else
+  {
+    btype = get_type_at_level(v, 0);
+    if (btype->ty_code == DT_ARRAY_OF) 
+      len = typesize(btype) - index_offset(dv);
+    else
+      len = typesize(btype);
+    if (len < 0)
+      len = UNKNOWN_SIZE;
+  }
+  if (len == UNKNOWN_SIZE)
+    errf("\bUnknown size");
+  else
+    errf("\bSize is %d bytes", len);
+}
+
+void
+remove_all_vars()
+{
+  objid_t obj, next_obj = NULL;
+  int oldstate;
+  
+  oldstate = td_set_obj_updating(OBJ_UPDATING_OFF);
+  for (obj = get_code(SRCHEAD_OBJCODE, OBJ_CHILD); obj != NULL;)
+  {
+    next_obj = get_code(obj, OBJ_NEXT);
+    remove_object(obj, OBJ_DESCENDENTS);
+    remove_object(obj, OBJ_SELF);
+    obj = next_obj;
+  }
+  td_set_obj_updating(oldstate);
+}
+
+
