@@ -35,6 +35,7 @@ char ups_va_expr_c_sccsid[] = "@(#)va_expr.c	1.26 24 May 1995 (UKC)";
 #include "exec.h"	/* for SET_EXPR_RESULT_ADDR */
 #include "obj_stack.h"
 #include "obj_util.h"
+#include "obj_misc.h"
 #include "menudata.h"
 #include "va_priv.h"
 #include "tdr.h"
@@ -612,6 +613,9 @@ char *arg;
 	case MR_VAR_DUMP_TO_OUTWIN:
 		dump_de_to_outwin(de);
 		break;
+	case MR_VAR_SHOW_SIZE:
+	        errf("Cannot show size of an expression");
+		break;
 	case MR_VAR_DUP:
 		add_to_new_selection((objid_t)dup_dexpr(de));
 	    	note_var_change((objid_t)de);
@@ -635,8 +639,12 @@ char *arg;
 		 *  is supplied by the user.
 		 */
 		break;
+	case MR_VAR_ACCELERATOR_ACTION:
+		/*  if case va_accelerator_action_from_type fails
+		 */
+		break;
 	default:
-		panic("bad cmd in de");
+		errf("bad cmd in de"); /* RGA from panic */
 	}
 }
 
@@ -1131,4 +1139,3 @@ dexpr_t *de;
     sprintf(buff,"%%d%c %lx %d", grpchar,addr,length);
     dump_memory (buff);
 }
-
