@@ -707,6 +707,10 @@ bool restoring;
 	if (v->va_name == NULL)
 		v->va_name = "unnamed";
 	dv->dv_var = v;
+	if (v)			/* RGA for changing targets */
+	  dv->dv_va_name = strdup(v->va_name);
+	else
+	   dv->dv_va_name = NULL;
 	dv->dv_ilevel = default_level(v->va_type, v->va_language);
 	dv->dv_ilist = make_ilist(v->va_type, v->va_language, dv->dv_ilevel);
 	dv->dv_addr = get_dv_addr(par, v, dv->dv_ilist, &junk);
@@ -936,6 +940,9 @@ vformat_t  fmt;
     int issigned = 0;
     type_t* base_type;
 
+    if (type == NULL)		/* RGA */
+      return MR_VAR_ACCELERATOR_ACTION;
+
     /*
     **   Determine the most useful accelerator when a given
     **   data type is double clicked on
@@ -1424,6 +1431,9 @@ char *arg;
 	case MR_VAR_DUMP_TO_OUTWIN:
 		dump_dv_to_outwin(dv);
 		break;
+	case MR_VAR_SHOW_SIZE:
+		show_dv_size(dv);
+		break;
 	case MR_VAR_DUP:
 		dv2 = new_dvar();
 		*dv2 = *dv;
