@@ -80,6 +80,7 @@ typedef struct vfile_s {
 	int objtype;
 	Velist *velist;
 	bool been_displayed;
+	bool been_restored;
 	bool user_changed_vars;
 	struct vfile_s *next;
 } Vfile;
@@ -124,7 +125,7 @@ static int load_velist PROTO((FILE *fp, int line_offset, int *p_lnum,
 			      alloc_pool_t **p_ap, Velist **p_velist));
 static Velist *reverse_velist PROTO((Velist *velist));
 static bool update_stack_entry_if_needed PROTO((Stack *stk, bool force));
-static void update_vf PROTO((Vfile *vf));
+static void update_vf PROTO((Vfile *vf, bool just_started, bool set_been_displayed));
 
 static Vstack *Vslist = NULL;
 static Vfile *Vflist = NULL;
@@ -198,6 +199,7 @@ Velist *velist;
 	vf->next = Vflist;
 	vf->user_changed_vars = FALSE;
 	vf->been_displayed = FALSE;
+	vf->been_restored = FALSE;
 	Vflist = vf;
 
 	return vf;
@@ -231,7 +233,10 @@ Velist *velist;
 	vs->func = f;
 	vs->velist = velist;
 	vs->updated = FALSE;
-	vs->been_displayed = FALSE;
+	/*	vs->been_displayed = FALSE;*/
+	/* RGA Changing this so stack variables are reinstated on repeated
+	   attach/detach cycles, as well as start/kill cycles */
+	vs->been_displayed = TRUE;
 	vs->next = Vslist;
 	Vslist = vs;
 
@@ -437,7 +442,9 @@ objid_t par;
 }
 
 void
-save_var_display_state()
+save_var_display_state(just_started, set_been_displayed)
+bool just_started;
+bool set_been_displayed;
 {
 	Vstack *vs;
 	Vfile *vf;
@@ -452,14 +459,16 @@ save_var_display_state()
 		vs->updated = TRUE;
 
 	for (vf = Vflist; vf != NULL; vf = vf->next)
-		update_vf(vf);
+		update_vf(vf, just_started, set_been_displayed);
 }
 
 static void
-update_vf(vf)
+update_vf(vf, just_started, set_been_displayed)
 Vfile *vf;
+bool just_started;
+bool set_been_displayed;
 {
-	if (vf->user_changed_vars) {
+        if (vf->user_changed_vars) {
 		Velist *velist;
 		
 		alloc_reset_pool(vf->apool);
@@ -469,8 +478,13 @@ Vfile *vf;
 		vf->velist = reverse_velist(velist);
 		
 		vf->user_changed_vars = FALSE;
-		vf->been_displayed = TRUE;
+		if (just_started)
+		  vf->been_displayed = TRUE;
 	}
+	else
+	  if (set_been_displayed)
+	    vf->been_displayed = TRUE;
+
 }
 
 void
@@ -480,7 +494,7 @@ objid_t obj;
 	Vfile *vf;
 
 	if ((vf = obj_to_vfile(obj)) != NULL)
-		update_vf(vf);
+		update_vf(vf, TRUE, FALSE);
 }
 
 void
@@ -489,7 +503,6 @@ restore_file_displays()
 	Vfile *vf;
 	
 	for (vf = Vflist; vf != NULL; vf = vf->next) {
-/*	RGA	if (!vf->been_displayed)*/
 		if (vf->been_displayed)
 			continue;
 
@@ -498,6 +511,9 @@ restore_file_displays()
 			add_common_block_object_if_necessary((common_block_t *)vf->obj);
 			break;
 		case OT_SFILE:
+		  	if (((fil_t *)(vf->obj))->fi_symtab &&
+			    ((fil_t *)(vf->obj))->fi_symtab->st_eclipsed)
+			  continue;
 			add_source_file_object_if_necessary((fil_t *)vf->obj, FALSE);
 			break;
 		case OT_GLOBALS:
@@ -508,7 +524,8 @@ restore_file_displays()
 		}
 	
 		restore_display(vf->obj, vf->velist);
-		vf->user_changed_vars = FALSE;
+		/*	RGA always want an update for multiple attach/detach cycles */
+		/*		vf->user_changed_vars = FALSE;*/
 	}
 }
 
@@ -521,8 +538,7 @@ bool force;
 	
 	vf = obj_to_vfile(obj);
 
-/* RGA	if (vf != NULL && (force || vf->been_displayed)) {*/
-	if (vf != NULL && (force || !vf->been_displayed)) {
+ 	if (vf != NULL && (force || vf->been_displayed)) {
 		restore_display(vf->obj, vf->velist);
 		vf->been_displayed = TRUE;
 	}
@@ -530,6 +546,25 @@ bool force;
 	return vf != NULL;
 }
 
+bool
+restore_files(obj)
+     objid_t obj;
+{
+  Vfile *vf;
+	
+  vf = obj_to_vfile(obj);
+
+  if (vf != NULL && !vf->been_restored)
+  {
+    restore_display(vf->obj, vf->velist);
+    if (vf->objtype == OT_SFILE || vf->objtype == OT_GLOBALS)
+      add_source_file_object_if_required((fil_t *)obj, FALSE, FALSE, TRUE);
+    vf->been_restored = TRUE;
+  }
+
+  return vf != NULL;
+}
+
 void
 note_file_collapsed(obj, dummy)
 objid_t obj;
@@ -539,6 +574,7 @@ bool dummy;
 	
 	if ((vf = obj_to_vfile(obj)) != NULL) {
 		vf->been_displayed = FALSE;
+		vf->been_restored = FALSE;
 		vf->user_changed_vars = FALSE;
 	}
 }
@@ -683,7 +719,7 @@ objid_t par;
 Vsvar *vv;
 int block_start, block_end;
 {
-	objid_t obj;
+	objid_t obj = NULL;
 	
 	switch (ups_get_object_type(par)) {
 	case OT_FUNC:
@@ -692,6 +728,9 @@ int block_start, block_end;
 		break;
 		
 	case OT_SFILE:
+	  	if (((fil_t *)par)->fi_symtab &&
+		    ((fil_t *)par)->fi_symtab->st_eclipsed)
+		  break;
 		show_global((Srcwin *)NULL, (fil_t *)par, (func_t *)NULL,
 			    (common_block_t *)NULL,
 			    vv->name, FALSE, FALSE, TRUE, &obj, FALSE);
@@ -814,13 +853,16 @@ objid_t par;
 Vsexpr *vc;
 int block_start, block_end;
 {
-	block_t *rootbl, *bl;
+	block_t *rootbl = NULL, *bl;
 	
 	switch (ups_get_object_type(par)) {
 	case OT_FUNC:
 		rootbl = FU_BLOCKS(((Stack *)par)->stk_func);
 		break;
 	case OT_SFILE:
+	  	if (((fil_t *)par)->fi_symtab &&
+		    ((fil_t *)par)->fi_symtab->st_eclipsed)
+		  break;
 		rootbl = ((fil_t *)par)->fi_block;
 		break;
 	default:
@@ -901,6 +943,9 @@ FILE *fp;
 			break;
 
 		case OT_SFILE:
+		  	if (((fil_t *)(vf->obj))->fi_symtab &&
+			    ((fil_t *)(vf->obj))->fi_symtab->st_eclipsed)
+			  continue;
 			fprintf(fp, "file %s", ((fil_t *)vf->obj)->fi_name);
 			break;
 
@@ -1486,3 +1531,42 @@ int *p_lnum;
 
 	return res;
 }
+
+void
+reset_file_restore_states()
+{
+  Vfile *vf;
+  
+  for (vf = Vflist; vf != NULL; vf = vf->next) 
+      vf->been_restored = FALSE;
+}
+
+void
+free_vslist()
+{
+  Vstack *vs, *vs_next;
+
+  for (vs = Vslist; vs != NULL;)
+  {
+    vs_next = vs->next;
+    alloc_free_pool(vs->apool);
+    free(vs);
+    vs = vs_next;
+  }
+  Vslist = NULL;
+}
+
+void
+free_vflist()
+{
+  Vfile *vf, *vf_next;
+  
+  for (vf = Vflist; vf != NULL;)
+  {
+    vf_next = vf->next;
+    alloc_free_pool(vf->apool);
+    free(vf);
+    vf = vf_next;
+  }
+  Vflist = NULL;
+}
