@@ -826,7 +826,7 @@ dvar_t *dv;
 		break;
 
 	default:
-		strcpy(buf, "<unknown type - tell mtr@ukc.ac.uk>");
+		strcpy(buf, "<unknown type>");
 	}
 
 	if (*buf == '\0')
